/*!
 * \file AudioPlayerUtil.m
 *      The utility for AVAudioPlayer library
 */
/***************************************************************************/
/* Copyright (c) 2009 Aurora Wand Studio                                   */
/* All rights reserved.                                                    */
/*                                                                         */
/* No part of this document may be copied, reproduced,                     */
/* or transmitted in any form without the prior written                    */
/* consent of Aurora Wand Studio, San Jose, CA                             */
/*                                                                         */
/* Created by LinCheng Wang on 11/02/09                                    */
/***************************************************************************/

/****************************************************************************
 * COMPILATION OPTIONS
 ***************************************************************************/

/****************************************************************************
 * HEADER FILES
 ***************************************************************************/
#import "AudioPlayerUtil.h"

/****************************************************************************
 * MACROS and INLINE FUNCTIONS
 ***************************************************************************/

/****************************************************************************
 * TYPEDEF and DATA STRUCTURE
 ***************************************************************************/

/****************************************************************************
 * EXTERNAL, GLOBAL, and LOCAL VARIABLES
 ***************************************************************************/

/****************************************************************************
 * EXTERNAL and PRIVATE FUNCTIONS
 ***************************************************************************/

/****************************************************************************
 * CLASS IMPLEMENTATION - AVAudioPlayer (Utility)
 ***************************************************************************/
@implementation AVAudioPlayer (Utility)

/****************************************************************************
 * METHODS - AVAudioPlayer (Utility)
 ***************************************************************************/
//---------------------------------------------------------------------------
/*
 * AVAudioPlayer Class Reference
 *
 * Initializing an AVAudioPlayer Object
 *      - initWithContentsOfURL:error
 *              Initializes and returns an audio player for playing a designated sound file.
 *      - initWithData:error:
 *              Initializes and returns an audio player for playing a designated memory buffer.
 *
 * Configuring and Controlling Playback
 *      - play: Plays a sound asynchronously
 *      - pause: Pauses playback; sound remains ready to resume playback from where it left off
 *      - stop: Stops playback and undoes the setup needed for playback
 *              It does not reset the value of the currentTime property to 0.
 *
 *      - prepareToPlay - Prepares the audio player for playback by preloading its buffers
 *        playing (property) - (read-only)
 *              A Boolean value that indicates whether the audio player is playing (YES) or not (NO).
 *        volume (property) -
 *              The playback gain for the audio player, ranging from 0.0 through 1.0.
 *        numberOfLoops (property) -
 *              The number of times a sound will return to the beginning, upon reaching the end, to repeat playback.
 *        delegate (property) - The object that you assign to be an audio player’s delegate becomes
 *              the target of the notifications
 *
 * Managing Information About a Sound
 *        numberOfChannels (property) - (ready-only)
 *              The number of audio channels in the sound associated with the audio player
 *        duration (property) - (read-only)
 *              Returns the total duration, in seconds, of the sound associated with the audio player.
 *        currentTime (property) -
 *              The playback point, in seconds, within the timeline of the sound
 *              associated with the audio player.
 *        url (property) - (read-only)
 *              The URL for the sound associated with the audio player.
 *        data (property) - (read-only)
 *              The data object containing the sound associated with the audio player.
 *              Returns nil if the audio player has no data (that is, if it was not initialized with an NSData object).
 *
 * Using Audio Level Metering
 *        meteringEnabled (property):
 *              A Boolean value that indicates the audio-level metering on/off state for the audio player.
 *      - averagePowerForChannel:
 *              Returns the average power for a given channel, in decibels, for the sound being played.
 *      - peakPowerForChannel:
 *              Returns the peak power for a given channel, in decibels, for the sound being played.
 *      - updateMeters
 *              Refreshes the average and peak power values for all channels of an audio player.
 */

/*
 * AVAudioPlayerDelegate Protocol Reference
 *
 * The delegate of an AVAudioPlayer object must adopt the AVAudioPlayerDelegate protocol.
 * All of the methods in this protocol are optional.
 * They allow a delegate to respond to audio interruptions and audio decoding errors,
 * and to the completion of a sound’s playback.
 *
 *  Responding to Sound Playback Completion
 *      – audioPlayerDidFinishPlaying:successfully:  required method
 *              Called when a sound has finished playing.
 *              This method is not called upon an audio interruption.
 *              Rather, an audio player is paused upon interruption—the sound has not finished playing.
 *  Responding to an Audio Decoding Error
 *      – audioPlayerDecodeErrorDidOccur:error:  required method
 *              Called when an audio player encounters a decoding error during playback.
 *
 *  Handling Audio Interruptions
 *      – audioPlayerBeginInterruption:  required method
 *              Called when an audio player is interrupted, such as by an incoming phone call.
 *              Upon interruption, your application’s audio session is deactivated and the audio player pauses.
 *              You cannot use the audio player again until you receive a notification that the interruption has ended.
 *
 *      – audioPlayerEndInterruption:  required method
 *              Called when an interruption ends, such as by a user ignoring an incoming phone call. 
 *              When an interruption ends, the audio session for your application is automatically reactivated,
 *              allowing you to interact with the audio player. To resume playback, call the play method.
 */
//---------------------------------------------------------------------------
/*! \fn + (id)createAudioPlayerWithPath:(NSString *)psPath ofType:(NSString *)psExt
 *  \brief Allocate and initialize an AudioPlayer
 */
+ (id)createAudioPlayerWithPath:(NSString *)psPath ofType:(NSString *)psExt
{
    NSBundle *pMainBundle = [NSBundle mainBundle];
    NSString *psSoundFile = [pMainBundle pathForResource:psPath ofType:psExt];
    NSURL    *pURL        = [NSURL fileURLWithPath:psSoundFile];

/* initialize with URL file */
    /* Initializes and returns an audio player for playing a designated sound file */
    if((self = [[AVAudioPlayer alloc] initWithContentsOfURL:pURL error:nil]) == nil)
        return self;

/* set defaults */
#if 0
    /* Prepares the audio player for playback by preloading its buffers
     *
     * Calling this method preloads buffers and acquires the audio hardware needed for playback,
     * which minimizes the lag between calling the play method and the start of sound output.
     *
     * Calling the stop method, or allowing a sound to finish playing, undoes this setup.
     */
    [self prepareToPlay];

    /* The playback gain for the audio player, ranging from 0.0 through 1.0. */
    [self setVolume:1.0];

    /* start at beginning */
    [self setCurrentTime:0];

    /* The object that you assign to be an audio player’s delegate becomes
     * the target of the notifications described in AVAudioPlayerDelegate Protocol Reference.
     * These notifications let you respond to decoding errors, audio interruptions
     * (such as an incoming phone call), and playback completion.
     */ 
    [self setDelegate:self];
#endif
/* return */
    return self;
}

//---------------------------------------------------------------------------
/*! \fn - (void)playContinuously
 *  \brief Play continuously
 */
- (void)playContinuously
{
    [self setNumberOfLoops:-1];
}

//---------------------------------------------------------------------------
/* the following code example should be added to calling files
 */
#if 0
#pragma mark AV Foundation delegate methods____________

- (void) audioPlayerDidFinishPlaying: (AVAudioPlayer *) appSoundPlayer successfully: (BOOL) flag {

	playing = NO;
	[appSoundButton setEnabled: YES];
}

- (void) audioPlayerBeginInterruption: player {

	NSLog (@"Interrupted. The system has paused audio playback.");
	
	if (playing) {
	
		playing = NO;
		interruptedOnPlayback = YES;
	}
}

- (void) audioPlayerEndInterruption: player {

	NSLog (@"Interruption ended. Resuming audio playback.");
	
	// Reactivates the audio session, whether or not audio was playing
	//		when the interruption arrived.
	[[AVAudioSession sharedInstance] setActive: YES error: nil];
	
	if (interruptedOnPlayback) {
	
		[appSoundPlayer prepareToPlay];
		[appSoundPlayer play];
		playing = YES;
		interruptedOnPlayback = NO;
	}
}

#endif

/*-------------------------------------------------------------------------*/
@end  /* end of AVAudioPlayer (Utility) */
