/*!
 * \file BookDetailViewCtrl.h
 *      Implement BookDetail View Controller for BuddhaCounter project
 *
 * The table view controller responsible for displaying detailed
 * information about a single book.
 * It also allows the user to edit information about a book,
 * and supports undo for editing operations.
 * 
 * When editing begins, the controller creates and set
 * an undo manager to track edits.
 * It then registers as an observer of undo manager change notifications,
 * so that if an undo or redo operation is performed,
 * the table view can be reloaded.
 * When editing ends, the controller de-registers from the notification
 * center and removes the undo manager.
 */
/***************************************************************************/
/* Copyright (c) 2010 AuroraWand LLC                                       */
/* All rights reserved.                                                    */
/*                                                                         */
/* No part of this document may be copied, reproduced,                     */
/* or transmitted in any form without the prior written                    */
/* consent of AuroraWand LLC, San Jose, CA                                 */
/*                                                                         */
/* Created by LinCheng Wang on 05/25/10                                    */
/***************************************************************************/

//=============================================================================
// COMPILATION OPTIONS
//=============================================================================
//#define DEBUG_LEVEL     /* to turn on message utility by UtilMsg.h */

//=============================================================================
// HEADER FILES
//=============================================================================
#import "EditingViewCtrl.h"

//=============================================================================
// MACROS & INLINE FUNCTIONS
//=============================================================================

//=============================================================================
// TYPEDEFS & DATA STRUCTURES
//=============================================================================
typedef enum {
    eEditMode_View, /* no change */
    eEditMode_Add,  /* allow to change most of items */
} EEditMode;

//=============================================================================
// CLASS INTERFACE
//=============================================================================
@class Book;
@protocol BookDetailViewCtrlDelegate;

@interface BookDetailViewCtrl : UIViewController <UITextFieldDelegate>
{
    id <BookDetailViewCtrlDelegate> bookDetailViewCtrlDelegate;
/* User Interface */
    UIImageView         *xpBgView;
    UISegmentedControl  *xpSegCategory;
    UITextField         *xpFldTitle;
    UITextField         *xpFldCountTarget;
    UILabel             *xpLblCountNow;
    UILabel             *xpLblPercentComplete;
    UILabel             *xpLblCountTotal;
    UILabel             *xpLblDateLastRead;
    UILabel             *xpLblDateCreate;

    UILabel             *xpNameCategory;
    UILabel             *xpNameTitle;
    UILabel             *xpNameCountTarget;
    UILabel             *xpNameCountNow;
    UILabel             *xpNamePercentComplete;
    UILabel             *xpNameCountTotal;
    UILabel             *xpNameDateCreate;
    UILabel             *xpNameDateLastRead;

	UIBarButtonItem     *mpDoneButton;

/* data model */
    Book            *book;
	NSDateFormatter *dateFormatter;
	//NSUndoManager   *undoManager;

/* variables */
    EEditMode        eEditMode;
    BOOL            mbIsChanged;
}

@property (nonatomic, assign) id <BookDetailViewCtrlDelegate> bookDetailViewCtrlDelegate;

/* User Interface */
@property (nonatomic, retain) IBOutlet UIImageView         *xpBgView;
@property (nonatomic, retain) IBOutlet UISegmentedControl  *xpSegCategory;
@property (nonatomic, retain) IBOutlet UITextField         *xpFldTitle;
@property (nonatomic, retain) IBOutlet UITextField         *xpFldCountTarget;
@property (nonatomic, retain) IBOutlet UILabel             *xpLblCountNow;
@property (nonatomic, retain) IBOutlet UILabel             *xpLblPercentComplete;
@property (nonatomic, retain) IBOutlet UILabel             *xpLblCountTotal;
@property (nonatomic, retain) IBOutlet UILabel             *xpLblDateLastRead;
@property (nonatomic, retain) IBOutlet UILabel             *xpLblDateCreate;

@property (nonatomic, retain) IBOutlet UILabel             *xpNameCategory;
@property (nonatomic, retain) IBOutlet UILabel             *xpNameTitle;
@property (nonatomic, retain) IBOutlet UILabel             *xpNameCountTarget;
@property (nonatomic, retain) IBOutlet UILabel             *xpNameCountNow;
@property (nonatomic, retain) IBOutlet UILabel             *xpNamePercentComplete;
@property (nonatomic, retain) IBOutlet UILabel             *xpNameCountTotal;
@property (nonatomic, retain) IBOutlet UILabel             *xpNameDateLastRead;
@property (nonatomic, retain) IBOutlet UILabel             *xpNameDateCreate;

@property (nonatomic, retain) IBOutlet UIBarButtonItem     *mpDoneButton;

/* data model */
@property (nonatomic, retain)          Book            *book;
@property (nonatomic, retain)          NSDateFormatter *dateFormatter;
//@property (nonatomic, retain)          NSUndoManager   *undoManager;

/* variables */
@property (nonatomic, assign)          EEditMode        eEditMode;

//=============================================================================
// METHODS
//=============================================================================
- (IBAction)onSegCategory;
- (IBAction)onDone;
- (IBAction)onBack;
- (IBAction)onTextFieldDoneEditing:(UITextField *)pTextField;

//- (void)setUpUndoManager;
//- (void)cleanUpUndoManager;
- (void)updateRightBarButtonItemState;

//=============================================================================
@end    /* BookDetailViewCtrl */

//=============================================================================
// PROTOCOL METHODS
//=============================================================================
@protocol BookDetailViewCtrlDelegate
- (void)bookDetailViewCtrl:(BookDetailViewCtrl *)pViewCtrl didFinishWithSave:(BOOL)bIsSave;

//=============================================================================
@end    /* BookDetailViewCtrlDelegate */
