/*!
 * \file BookDetailViewCtrl.m
 *      Implement BookDetail View Controller for BuddhaCounter project
 *
 * The table view controller responsible for displaying detailed
 * information about a single book.
 * It also allows the user to edit information about a book,
 * and supports undo for editing operations.
 * 
 * When editing begins, the controller creates and set
 * an undo manager to track edits.
 * It then registers as an observer of undo manager change notifications,
 * so that if an undo or redo operation is performed,
 * the table view can be reloaded.
 * When editing ends, the controller de-registers from the notification
 * center and removes the undo manager.
 */
/***************************************************************************/
/* Copyright (c) 2010 AuroraWand LLC                                       */
/* All rights reserved.                                                    */
/*                                                                         */
/* No part of this document may be copied, reproduced,                     */
/* or transmitted in any form without the prior written                    */
/* consent of AuroraWand LLC, San Jose, CA                                 */
/*                                                                         */
/* Created by LinCheng Wang on 05/25/10                                    */
/***************************************************************************/

//=============================================================================
// COMPILATION OPTIONS
//=============================================================================
//#define DEBUG_LEVEL     /* to turn on message utility by UtilMsg.h */
#define DATE_REMOVED

#define USE_INTERFACE_BUILDER

//=============================================================================
// HEADER FILES
//=============================================================================
#import "BookDetailViewCtrl.h"
#import "EditingViewCtrl.h"

#import "AppModel.h"

//=============================================================================
// MACROS & INLINE FUNCTIONS
//=============================================================================

//=============================================================================
// TYPEDEFS & DATA STRUCTURES
//=============================================================================
#if 0
typedef enum {
    eFieldType_Category,
    eFieldType_Title,
    eFieldType_CountTarget,
    eFieldType_CountNow,
#ifdef MORE_DATE_FIELD
    eFieldType_DateStart,
    eFieldType_DateFinish,
#endif  /* MORE_DATE_FIELD */
    eFieldType_PercentComplete,
    eFieldType_DateLastRead,
    eFieldType_DateCreate,
    eFieldType_Num
} EFieldType;
#endif

//=============================================================================
// CLASS IMPLEMENTATION
//=============================================================================
@implementation BookDetailViewCtrl

@synthesize bookDetailViewCtrlDelegate;
@synthesize xpBgView;
@synthesize xpSegCategory;
@synthesize xpFldTitle, xpFldCountTarget;
@synthesize xpLblCountNow, xpLblPercentComplete,
            xpLblCountTotal,
            xpLblDateLastRead, xpLblDateCreate;
@synthesize xpNameCategory,
            xpNameTitle,
            xpNameCountTarget,
            xpNameCountNow,
            xpNamePercentComplete,
            xpNameCountTotal,
            xpNameDateCreate,
            xpNameDateLastRead;
@synthesize mpDoneButton;

/* AppSetting */
//@synthesize mpAppSetting;

@synthesize book, dateFormatter, eEditMode;
//@synthesize undoManager;

//=============================================================================
// METHODS
//=============================================================================
#pragma mark -
#pragma mark View lifecycle

//-----------------------------------------------------------------------------
- (void)viewDidLoad {
    [super viewDidLoad];
    
/* User Interface, hacked for XIB for initialization */

/* Configure the title, title bar, and table view. */
    self.title = NSLocalizedString(@"Book Details", @"the view title of showing book details view");
    for(ECategory eCategory = 0; eCategory < eCategory_Num; eCategory++) {
    	[xpSegCategory setTitle:[Book categoryForTitle:eCategory]
             forSegmentAtIndex:eCategory];
    }
    
    xpNameCategory.text        = NSLocalizedString(@"Category",         @"Name for Category");
    xpNameTitle.text           = NSLocalizedString(@"Title",            @"Name for Title");
    xpNameCountTarget.text     = NSLocalizedString(@"Target Count",     @"Name for Target Count");
    xpNameCountNow.text        = NSLocalizedString(@"Current Count",    @"Name for Current Count");
    xpNamePercentComplete.text = NSLocalizedString(@"Complete",         @"Name for Complete Percent");
    xpNameCountTotal.text      = NSLocalizedString(@"Total Count",      @"Name for Total Count");
    xpNameDateCreate.text      = NSLocalizedString(@"Created Date",     @"Name for Created Date");
    xpNameDateLastRead.text    = NSLocalizedString(@"Last Read Date",   @"Name for Last Read Date");

#if 1
    // Configure the save and cancel buttons.
	UIBarButtonItem *doneButton = [[UIBarButtonItem alloc]
        initWithBarButtonSystemItem:UIBarButtonSystemItemDone
                             target:self
                             action:@selector(onDone)];
    self.mpDoneButton = doneButton;
	[doneButton release];
	
	UIBarButtonItem *backButton = [[UIBarButtonItem alloc]
                                   	initWithTitle:NSLocalizedString(@"Back", @"navigation symbol of back to previous view")
                                            style:UIBarButtonItemStyleDone
                                           target:self
                                           action:@selector(onBack)];
	self.navigationItem.leftBarButtonItem = backButton;
	[backButton release];
#else
    self.navigationItem.rightBarButtonItem = self.editButtonItem;
#endif

    self.eEditMode = eEditMode_View;
    mbIsChanged = NO;
}


//-----------------------------------------------------------------------------
- (void)viewWillAppear:(BOOL)animated {
/* Redisplay the data. */
    xpSegCategory.selectedSegmentIndex = [book.category intValue];
    xpFldTitle.text                    = book.title;
    xpFldCountTarget.text              = [NSString stringWithFormat:@"%d", [book.countTarget intValue]];
    xpLblCountNow.text                 = [NSString stringWithFormat:@"%d", [book.countNow intValue]];
    xpLblPercentComplete.text          = [NSString stringWithFormat:@"%6.2f%%", [book.percentComplete floatValue]];
    xpLblCountTotal.text               = [NSString stringWithFormat:@"%d", [book.countTotal intValue]];
    xpLblDateLastRead.text             = [self.dateFormatter stringFromDate:book.dateLastRead];
    xpLblDateCreate.text               = [self.dateFormatter stringFromDate:book.dateCreate];

/* update save/edit button */
    [self updateRightBarButtonItemState];
}

//-----------------------------------------------------------------------------
- (void)setEditing:(BOOL)editing animated:(BOOL)animated {
    [super setEditing:editing animated:animated];

/* Hide the back button when editing starts, and show it again when editing finishes. */
    [self.navigationItem setHidesBackButton:editing animated:animated];
    //TODO: [self.xpTableView reloadData];

/* When editing starts, create and set an undo manager to track edits.
 * Then register as an observer of undo manager change notifications,
 * so that if an undo or redo operation is performed,
 * the table view can be reloaded.
 * When editing ends, de-register from the notification center and
 * remove the undo manager, and save the changes.
 */
    if (editing) {
        //[self setUpUndoManager];
    } else {
        //[self cleanUpUndoManager];
        // Save the changes.
        NSError *error;
        if (![book.managedObjectContext save:&error]) {
#ifdef DEVELOPMENT_ONLY
            // Update to handle the error appropriately.
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            exit(-1);  // Fail
#endif
        }
    }
}

//-----------------------------------------------------------------------------
- (void)viewDidUnload {
/* Release any properties that are loaded in viewDidLoad or can be recreated lazily. */
/* data model */
    self.dateFormatter = nil;

/* user interface */
    self.xpBgView             = nil;
    self.xpSegCategory        = nil;
    self.xpFldTitle           = nil;
    self.xpFldCountTarget     = nil;
    self.xpLblCountNow        = nil;
    self.xpLblPercentComplete = nil;
    self.xpLblCountTotal      = nil;
    self.xpLblDateLastRead    = nil;
    self.xpLblDateCreate      = nil;

    self.xpNameCategory.text        = nil;
    self.xpNameTitle.text           = nil;
    self.xpNameCountTarget.text     = nil;
    self.xpNameCountNow.text        = nil;
    self.xpNamePercentComplete.text = nil;
    self.xpNameCountTotal.text      = nil;
    self.xpNameDateCreate.text      = nil;
    self.xpNameDateLastRead.text    = nil;

    self.mpDoneButton         = nil;
}

//-----------------------------------------------------------------------------
/*! The view controller must be first responder in order to be able to
 * receive shake events for undo.
 * It should resign first responder status when it disappears.
 */
- (BOOL)canBecomeFirstResponder {
    return YES;
}


#if 0   /* not used in this project */
//-----------------------------------------------------------------------------
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    [self becomeFirstResponder];
}

//-----------------------------------------------------------------------------
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    //[self resignFirstResponder];

}
#endif

//-----------------------------------------------------------------------------
- (void)updateRightBarButtonItemState {
/* Conditionally enable the right bar button item --
 * it should only be enabled if the book is in a valid state for saving.
 */
/* TODO: why does it return NO? */
#if 0
    self.navigationItem.rightBarButtonItem.enabled = [book validateForUpdate:NULL];
#endif
}    

//=============================================================================
// PROTOCOL METHODS
//=============================================================================

#if 0
#pragma mark -
#pragma mark Undo support

//-----------------------------------------------------------------------------
- (void)setUpUndoManager {
    /*
     If the book's managed object context doesn't already have an undo manager, then create one and set it for the context and self.
     The view controller needs to keep a reference to the undo manager it creates so that it can determine whether to remove the undo manager when editing finishes.
     */
    if (book.managedObjectContext.undoManager == nil) {
        
        NSUndoManager *anUndoManager = [[NSUndoManager alloc] init];
        [anUndoManager setLevelsOfUndo:3];
        self.undoManager = anUndoManager;
        [anUndoManager release];
        
        book.managedObjectContext.undoManager = undoManager;
    }
    
    // Register as an observer of the book's context's undo manager.
    NSUndoManager *bookUndoManager = book.managedObjectContext.undoManager;
    
    NSNotificationCenter *dnc = [NSNotificationCenter defaultCenter];
    [dnc addObserver:self selector:@selector(undoManagerDidUndo:) name:NSUndoManagerDidUndoChangeNotification object:bookUndoManager];
    [dnc addObserver:self selector:@selector(undoManagerDidRedo:) name:NSUndoManagerDidRedoChangeNotification object:bookUndoManager];
}


//-----------------------------------------------------------------------------
- (void)cleanUpUndoManager {
    
    // Remove self as an observer.
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    if (book.managedObjectContext.undoManager == undoManager) {
        book.managedObjectContext.undoManager = nil;
        self.undoManager = nil;
    }        
}


//-----------------------------------------------------------------------------
- (NSUndoManager *)undoManager {
    return book.managedObjectContext.undoManager;
}


//-----------------------------------------------------------------------------
- (void)undoManagerDidUndo:(NSNotification *)notification {
    //[self.xpTableView reloadData];
    [self updateRightBarButtonItemState];
}


//-----------------------------------------------------------------------------
- (void)undoManagerDidRedo:(NSNotification *)notification {
    //[self.xpTableView reloadData];
    [self updateRightBarButtonItemState];
}
#endif


#pragma mark -
#pragma mark Date Formatter

//-----------------------------------------------------------------------------
- (NSDateFormatter *)dateFormatter {    
    if (dateFormatter == nil) {
        dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setDateStyle:NSDateFormatterMediumStyle];
        [dateFormatter setTimeStyle:NSDateFormatterNoStyle];
    }
    return dateFormatter;
}


#pragma mark -
#pragma mark Memory management

//-----------------------------------------------------------------------------
- (void)dealloc
{
/* data model */
    //[undoManager release];
    [dateFormatter release];
    [book release];

/* user interface */
    [xpBgView              release];

    [xpSegCategory         release];
    [xpFldTitle            release];
    [xpFldCountTarget      release];
    [xpLblCountNow         release];
    [xpLblPercentComplete  release];
    [xpLblCountTotal       release];
    [xpLblDateLastRead     release];
    [xpLblDateCreate       release];

    [xpNameCategory        release];
    [xpNameTitle           release];
    [xpNameCountTarget     release];
    [xpNameCountNow        release];
    [xpNamePercentComplete release];
    [xpNameCountTotal      release];
    [xpNameDateCreate      release];
    [xpNameDateLastRead    release];

    [mpDoneButton          release];

/* release super at last */
    [super dealloc];
}

//=============================================================================
// ACTION METHODS
//=============================================================================
//-----------------------------------------------------------------------------
- (void)onSegCategory
{
    if([book.category intValue] != xpSegCategory.selectedSegmentIndex) {
		book.category = [NSNumber numberWithInt:xpSegCategory.selectedSegmentIndex];
        mbIsChanged = YES;
    }
}

//-----------------------------------------------------------------------------
- (IBAction)onTextFieldDoneEditing:(UITextField *)pTextField
{
    [pTextField resignFirstResponder];
}

#pragma mark -
#pragma mark Save and cancel operations

//-----------------------------------------------------------------------------
- (IBAction)onDone
{
    [self.xpFldCountTarget resignFirstResponder];
    [self.xpFldTitle resignFirstResponder];
}

//-----------------------------------------------------------------------------
- (IBAction)onBack {
    if(eEditMode == eEditMode_View) {
        [bookDetailViewCtrlDelegate bookDetailViewCtrl:self didFinishWithSave:mbIsChanged];
    }
    // Don't pass current value to the edited object, just pop.
    //[self.navigationController popViewControllerAnimated:YES];
}

//=============================================================================
// PROTOCOL METHODS
//=============================================================================
//===========================================================================
#pragma mark <UITextFieldDelegate> Methods

#define kNumMax     999999
#define kNumMin          1

#if 0   /* not used in this project */
//-----------------------------------------------------------------------------
/*! return NO to disallow editing
 *
 * Action 1 after clicking a textField.
 */
- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
    return YES;
}
#endif  /* not used in this project */

//-----------------------------------------------------------------------------
/*! became first responder
 * 
 * Action 2 after clicking a textField 
 */
- (void)textFieldDidBeginEditing:(UITextField *)textField
{
    if(eEditMode == eEditMode_View)
        [self.navigationItem setRightBarButtonItem:mpDoneButton animated:YES];

#if 0
    if(eEditMode == eEditMode_View) {
	    self.navigationItem.rightBarButtonItem.enabled = YES;
#endif
}

#if 0   /* not used in this project */
//-----------------------------------------------------------------------------
/*! return YES to allow editing to stop and to resign first responder status.
 * NO to disallow the editing session to end
 *
 * Action 2 after clicking a return 
 */
- (BOOL)textFieldShouldEndEditing:(UITextField *)textField
{
    return YES;
}
#endif  /* not used in this project */

//-----------------------------------------------------------------------------
/*! may be called if forced even if shouldEndEditing returns NO
 * (e.g. view removed from window) or endEditing:YES called
 *
 * Action 3 after clicking a return
 */
- (void)textFieldDidEndEditing:(UITextField *)textField
{
/* remove save button */
    //if(eEditMode == eEditMode_View)
	//    self.navigationItem.rightBarButtonItem.enabled = NO;
    if(eEditMode == eEditMode_View)
        [self.navigationItem setRightBarButtonItem:nil animated:NO];

/* save content */
    if(textField == xpFldTitle) {                   /* Title */
        if([xpFldTitle.text isEqualToString:book.title])
            return;
        mbIsChanged = YES;
        book.title = xpFldTitle.text;
    } else if(textField == xpFldCountTarget) {      /* countTarget */
        int nVal = [xpFldCountTarget.text intValue];
        if(nVal > kNumMax)
            nVal = kNumMax;
        else if(nVal < kNumMin)
            nVal = kNumMax;
        int nValOld = [book.countTarget intValue];
        if(nValOld == nVal)
            return;
        mbIsChanged = YES;
        book.countTarget      = [NSNumber numberWithInt:nVal];
        /* update percentComplete as well */
        const float fPercent  = ([book.countNow intValue] * 100.0) / nVal;
        book.percentComplete = [NSNumber numberWithFloat:fPercent];
        xpLblPercentComplete.text = [NSString stringWithFormat:@"%6.2f%%", fPercent];
    }
}

#if 0   /* not used in this project */
//-----------------------------------------------------------------------------
/*! return NO to not change text
 */
- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range
                                                       replacementString:(NSString *)string
{
}
#endif  /* not used in this project */

#if 0   /* not used in this project */
//-----------------------------------------------------------------------------
/*! called when clear button pressed.
 * return NO to ignore (no notifications)
 */
- (BOOL)textFieldShouldClear:(UITextField *)textField
{
}
#endif  /* not used in this project */

//-----------------------------------------------------------------------------
/*! called when 'return' key pressed. return NO to ignore.
 * 
 * Action 1 after clicking a return
 */
- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    return YES;
}

//=============================================================================
@end    /* BookDetailViewCtrl */
