/*!
 * \file  BookListViewCtrl.m
 *       Implement BookList View Controller for BuddhaCounter project
 *
 * The table view controller responsible for displaying the
 *           list of books, supporting additional functionality:
 * - Drill-down to display more information about a selected book using
 *   an instance of DetailViewCtrl;
 * - Addition of new new books;
 * - Deletion of existing books using UITableView's
 *   tableView:commitEditingStyle:forRowAtIndexPath: method.
 *
 * The root view controller creates and configures an instance of
 * NSFetchedResultsController to manage the collection of books. 
 * The view controller's managed object context is supplied by
 * the application's delegate.
 * When the user adds a new book, the root view controller creates a
 * new managed object context to pass to the add view controller;
 * this ensures that any changes made in the add controller do not affect
 * the main managed object context,
 * and they can be committed or discarded as a whole.
 */
/***************************************************************************/
/* Copyright (c) 2010 AuroraWand LLC                                       */
/* All rights reserved.                                                    */
/*                                                                         */
/* No part of this document may be copied, reproduced,                     */
/* or transmitted in any form without the prior written                    */
/* consent of AuroraWand LLC, San Jose, CA                                 */
/*                                                                         */
/* Created by LinCheng Wang on 05/25/10                                    */
/***************************************************************************/

//=============================================================================
// COMPILATION OPTIONS
//=============================================================================
//#define DEBUG_LEVEL     /* to turn on message utility by UtilMsg.h */

#define TABLE_VIEW_CELL_DESIGN

#define USE_INTERFACE_BUILDER

//=============================================================================
// HEADER FILES
//=============================================================================
#import "BookListViewCtrl.h"
#import "BookDetailViewCtrl.h"
#import "AppModel.h"
#import "AppSetting.h"

//=============================================================================
// MACROS & INLINE FUNCTIONS
//=============================================================================

//=============================================================================
// TYPEDEFS and DATA STRUCTURES
//=============================================================================

//=============================================================================
// PRIVATE METHODS
//=============================================================================
@interface BookListViewCtrl (Private)
- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath;
- (void)redoPerformFetch;
@end

//=============================================================================
// CLASS IMPLEMENTATION
//=============================================================================
@implementation BookListViewCtrl

@synthesize delegate;
@synthesize xpBgView, xpTableView, xpSortingKey, xpSortingLabel;
@synthesize fetchedResultsController, managedObjectContext, addingManagedObjectContext;
@synthesize mpAppSetting;

//=============================================================================
// METHODS
//=============================================================================
#pragma mark -
#pragma mark View lifecycle

//-----------------------------------------------------------------------------
#define kTableViewFrame     CGRectMake(0.0f, 0.0f, 320, 400)
#define kTableViewPos       CGPointMake(160, 200);

- (void)viewDidLoad
{
    [super viewDidLoad];

/* user interface */
    xpSortingLabel.text = NSLocalizedString(@"Sorting Selection", @"Label for Sorting Selection");
    
    for(ESortingKey eSortingKey = 0; eSortingKey < eSortingKey_Num; eSortingKey++) {
    	[xpSortingKey setTitle:[AppSetting sortingKeyInLocalizedString:eSortingKey]
             forSegmentAtIndex:eSortingKey];
    }
    
#ifdef USE_INTERFACE_BUILDER
    xpTableView.separatorColor  = [UIColor blackColor];
    xpTableView.backgroundColor = [UIColor clearColor];
#else
/* background */
    //UIImage *pBgImage = [UIImage imageNamed:@"BookListView_BG.png"];
    UIImage *pBgImage = [UIImage imageNamed:@"OptionView_BG.png"];
    UIImageView *pBgView = [[UIImageView alloc] initWithImage:pBgImage];
    pBgView.alpha = 1.0;
    self.xpBgView = pBgView;
    [pBgView release];
    [self.view addSubview:xpBgView];
    
/* tableView */
    /* TableView */
    /* initialize a UITableView and set its size */
    UITableView *pTableView = [[UITableView alloc] initWithFrame:kTableViewFrame style:UITableViewStyleGrouped];
    self.xpTableView = pTableView;
    [pTableView release];
    xpTableView.center = kTableViewPos;
    /* set background clearcolor to see thru background image */
    xpTableView.backgroundColor = [UIColor clearColor];
    /* set seapartor line to dark color */
    xpTableView.separatorColor = [UIColor blackColor];

    /* transform, if it is in landscape mode */
    //xpTableView.transform = CGAffineTransformIdentity;
    //xpTableView.transform = CGAffineTransformMakeRotation(1.57); //degreesToRadian(90)
    xpTableView.dataSource              = self;
    xpTableView.delegate                = self;
    //xpTableView.showsSelectionIndicator = YES;
    //[xpTableView setSoundsEnabled:YES];

/* add to its parent view (UIWindow) */
    [self.view addSubview:xpTableView];
#endif
    
/* make up View */
    /* Set the title. */
    self.title = NSLocalizedString(@"Book List", @"the view title of listing books view");
    //self.view.backgroundColor = [UIColor viewFlipsideBackgroundColor];  /* TODO */

    /* Set up the edit and add buttons.
     * The default button action invokes the setEditing:animated: method.
     */
    //self.navigationItem.leftBarButtonItem = self.editButtonItem;

    /* Configure the Done button. It is an action to back to previous view controller. */
    UIBarButtonItem *pBtnDone = [[UIBarButtonItem alloc]
        initWithBarButtonSystemItem:UIBarButtonSystemItemDone
                             target:self
                             action:@selector(onDone)];
    self.navigationItem.leftBarButtonItem = pBtnDone;
    [pBtnDone release];

    /* Configure the add button. */
#if 1
	UIBarButtonItem *pBtnAddBook = [[UIBarButtonItem alloc]
                      initWithTitle:NSLocalizedString(@"Add", @"navigation symbol of adding a new book")
                                   style:UIBarButtonItemStyleDone
                                   target:self
                                   action:@selector(onAddBook)];
    
#else
    UIBarButtonItem *pBtnAddBook = [[UIBarButtonItem alloc]
        initWithBarButtonSystemItem:UIBarButtonSystemItemAdd
                             target:self
                             action:@selector(onAddBook)];
#endif
    self.navigationItem.rightBarButtonItem = pBtnAddBook;
    [pBtnAddBook release];
    
    
/* current setting */
    xpSortingKey.selectedSegmentIndex    = [mpAppSetting.sortingKey     intValue];

/* Executes the fetch request on the store to get objects.
 * Returns YES if successful or NO (and an error) if a problem occurred. 
 * An error is returned if the fetch request specified doesn't include
 * a sort descriptor that uses sectionNameKeyPath.
 * After executing this method, the fetched objects can be accessed
 * with the property 'fetchedObjects'
 */
    NSError *error;
    if(![[self fetchedResultsController] performFetch:&error]) {
#ifdef DEVELOPMENT_ONLY
        // Update to handle the error appropriately.
        NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        exit(-1);  // Fail
#endif
    }

/*  update flag */
    mbIsUpdated = NO;
}

//-----------------------------------------------------------------------------
- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self.xpTableView reloadData];
}


//-----------------------------------------------------------------------------
- (void)viewDidUnload
{
/* Release any properties that are loaded in viewDidLoad or can be recreated lazily. */
/* data model */
    self.fetchedResultsController = nil;

/* user interface */
    self.xpBgView        = nil;
    self.xpTableView     = nil;
    self.xpSortingKey    = nil;
}

#pragma mark -
#pragma mark Memory management

//-----------------------------------------------------------------------------
- (void)dealloc
{
/* data model */
    [fetchedResultsController release];
    [managedObjectContext release];
    [addingManagedObjectContext release];

/* user interface */
    [xpBgView        release];
    [xpTableView     release];
    [xpSortingKey    release];

/* release super at last */
    [super dealloc];
}

//==============================================================================
// PROTOCOL METHODS
//==============================================================================
#define kTitleLabelTag      1
#define kValue1LabelTag     2
#define kValue2LabelTag     3
#define kCountTotalLabelTag     4

//------------------------------------------------------------------------------
/*! a private method to configure a TableViewCell
 */
- (void)configureCell:(UITableViewCell *)pCell atIndexPath:(NSIndexPath *)indexPath
{
/* Get the book corresponding to the current index path and configure the table view cell. */
    Book *pBook = [fetchedResultsController objectAtIndexPath:indexPath];

#ifdef TABLE_VIEW_CELL_DESIGN
    /* set text by each row */
    UILabel *pTitleLabel      = (UILabel *)[pCell.contentView viewWithTag:kTitleLabelTag];
    UILabel *pValue1Label     = (UILabel *)[pCell.contentView viewWithTag:kValue1LabelTag];
    UILabel *pValue2Label     = (UILabel *)[pCell.contentView viewWithTag:kValue2LabelTag];
    UILabel *pCountTotalLabel = (UILabel *)[pCell.contentView viewWithTag:kCountTotalLabelTag];

    pTitleLabel.text = pBook.title; 
    pValue1Label.text = [NSString stringWithFormat:@"%d / %d", [pBook.countNow intValue], [pBook.countTarget intValue]];
    //pValue2Label.text = [NSString stringWithFormat:@"%6.2f%%",
    //    (float)[pBook.countNow intValue] / [pBook.countTarget intValue] * 100.0];
    pValue2Label.text = [NSString stringWithFormat:@"%3.0f%%", [pBook.percentComplete floatValue]];
    pCountTotalLabel.text = [NSString stringWithFormat:@"%d", [pBook.countTotal intValue]];

/* the current read book */
    if([mpAppSetting isReadBook:pBook]) {
        pTitleLabel.textColor = [UIColor blackColor];
        pTitleLabel.font = [UIFont fontWithName:@"Verdana-Bold" size:20];
        /* Jill: color for highlighted titl */
        pTitleLabel.textColor = [UIColor colorWithRed:0.3
                                                green:0.2
                                                 blue:0.1
                                                alpha:1.0];
    } else {
		pTitleLabel.textColor = [UIColor blackColor];
        pTitleLabel.font = [UIFont fontWithName:@"Verdana" size:20];
    }

#else
/* A date formatter for the creation date. */
    static NSDateFormatter *dateFormatter = nil;
    if(dateFormatter == nil) {
        dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setTimeStyle:NSDateFormatterMediumStyle];
        [dateFormatter setDateStyle:NSDateFormatterShortStyle];
    }

/* Configure the cell to show the book's title */
    cell.textLabel.text       = pBook.title;
    cell.detailTextLabel.text = [dateFormatter stringFromDate:pBook.dateLastRead];
    //cell.imageView.image      = pBook.thumbnail;
#endif
}

#pragma mark -
#pragma mark Table view data source methods

//------------------------------------------------------------------------------
/*! The data source methods are handled primarily by the fetch results controller
 */
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
/* a bug fix for iPhone OS 3.0 - for a single section table */
#ifdef __IPHONE_3_1
    NSUInteger count = [[self.fetchedResultsController sections] count];
    //NSLog(@"number of sections=%d", count);
    return (count == 0) ? 1 : count;
#else
    return [[fetchedResultsController sections] count];
#endif
}

//------------------------------------------------------------------------------
/*! Customize the number of rows in the table view.
 */
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    id <NSFetchedResultsSectionInfo> sectionInfo = [[fetchedResultsController sections] objectAtIndex:section];
    //NSLog(@"numberRow=%d, section=%d, name=%@", [sectionInfo numberOfObjects], section, [sectionInfo name]);
    return [sectionInfo numberOfObjects];
}

//------------------------------------------------------------------------------
/*! Customize the appearance of table view cells.
 */
- (UITableViewCell *)tableView:(UITableView *)tableView
         cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *psCellIdentifier = @"CellForBookList";

/* get or create cell */
    UITableViewCell *pCell = [tableView dequeueReusableCellWithIdentifier:psCellIdentifier];
    if(pCell == nil) {
        /* This is the color of the texts for counts */
        UIColor *pColor = [UIColor colorWithRed:0.3
                                          green:0.1
                                           blue:0.1
                                          alpha:1.0];
		UIColor *tColor =  [UIColor colorWithRed:0.3
                                          green:0.1
                                           blue:0.2
                                          alpha:1.0];
		
#ifdef TABLE_VIEW_CELL_DESIGN
        pCell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault
                                reuseIdentifier:psCellIdentifier] autorelease];
        /* This is the color of the TableCell */
        pCell.backgroundColor = [UIColor colorWithRed:0.5 green:0.5 blue:0.5 alpha:0.3];
        //pCell.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"BookListView_CellBG.png"]];

#define TITLE_FONT_SIZE        20
#define NUMBER_FONT_SIZE       12
#define NUMBER_POS_Y           34
#define NUMBER_HEIGHT_Y        16
        /* This is the settings of the Title */
        CGRect aTitleFrame = CGRectMake(10, 4, 300, 24);
        UILabel *pTitleLabel = [[UILabel alloc] initWithFrame:aTitleFrame];
        pTitleLabel.font = [UIFont fontWithName:@"Verdana" size:TITLE_FONT_SIZE];
        pTitleLabel.textAlignment = UITextAlignmentLeft;
        pTitleLabel.textColor = [UIColor blackColor];
        pTitleLabel.backgroundColor = [UIColor clearColor];
        pTitleLabel.tag = kTitleLabelTag;
        [pCell.contentView addSubview:pTitleLabel];
        [pTitleLabel release];

        /* Jill: update CGRectMake for the sizes and positions of text labels */
        /* This is the settings of the counts in numbers */
        CGRect aValue1Frame = CGRectMake( 10, NUMBER_POS_Y, 120, NUMBER_HEIGHT_Y);
        UILabel *pValue1Label = [[UILabel alloc] initWithFrame:aValue1Frame];
        pValue1Label.font = [UIFont fontWithName:@"Verdana" size:NUMBER_FONT_SIZE];
        pValue1Label.textAlignment = UITextAlignmentCenter;
        pValue1Label.tag = kValue1LabelTag;
        pValue1Label.textColor = pColor;
        pValue1Label.backgroundColor = [UIColor clearColor];
        [pCell.contentView addSubview:pValue1Label];
        [pValue1Label release];

        /* This is the settings of the counts in percentage */
        CGRect aValue2Frame = CGRectMake(120, NUMBER_POS_Y, 60, NUMBER_HEIGHT_Y);
        UILabel *pValue2Label = [[UILabel alloc] initWithFrame:aValue2Frame];
        pValue2Label.font = [UIFont fontWithName:@"Verdana" size:NUMBER_FONT_SIZE];
        pValue2Label.textAlignment = UITextAlignmentRight;
        pValue2Label.tag = kValue2LabelTag;
        pValue2Label.textColor = pColor;
        pValue2Label.backgroundColor = [UIColor clearColor];
        [pCell.contentView addSubview:pValue2Label];
        [pValue2Label release];

        /* This is the settings of the total count */
        CGRect aCountTotalFrame = CGRectMake(170, NUMBER_POS_Y, 120, NUMBER_HEIGHT_Y);
        UILabel *pCountTotalLabel = [[UILabel alloc] initWithFrame:aCountTotalFrame];
        pCountTotalLabel.font = [UIFont fontWithName:@"Verdana-Bold" size:NUMBER_FONT_SIZE];
        pCountTotalLabel.textAlignment = UITextAlignmentRight;
        pCountTotalLabel.tag = kCountTotalLabelTag;
        pCountTotalLabel.textColor = tColor;
        pCountTotalLabel.backgroundColor = [UIColor clearColor];
        [pCell.contentView addSubview:pCountTotalLabel];
        [pCountTotalLabel release];
#else
        pCell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle
                                reuseIdentifier:psCellIdentifier] autorelease];
#endif
        /* it enables a deletion option */
        pCell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    }

/* Configure the cell. */
    [self configureCell:pCell atIndexPath:indexPath];
    return pCell;
}

//------------------------------------------------------------------------------
/*! return the title for section header
 */
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
#if 1
/* TODO: Display the categorys' names as section headings. */
    id <NSFetchedResultsSectionInfo> sectionInfo = [[fetchedResultsController sections] objectAtIndex:section];
    //NSLog(@"title = %@ %@ %d", [sectionInfo name], [sectionInfo indexTitle], [sectionInfo.name intValue]);
    return [Book categoryInString:[sectionInfo.name intValue]];
#else
    return [[[fetchedResultsController sections] objectAtIndex:section] name];
#endif
}

//-----------------------------------------------------------------------------
/*! Handle deletion of a book.
 */
- (void)tableView:(UITableView *)tableView
         commitEditingStyle:(UITableViewCellEditingStyle)editingStyle
          forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if(editingStyle == UITableViewCellEditingStyleDelete) {
        [tableView beginUpdates];
        
        /* Delete the managed object at the given index path. */
        /* TODO: use managedObjectContext directly? */
        NSManagedObjectContext *pContext = [fetchedResultsController managedObjectContext];
        Book *pSelectedBook = (Book*)[fetchedResultsController objectAtIndexPath:indexPath];
#if 0
        if([mpAppSetting isReadBook:pSelectedBook]) {
            NSLog(@"it is the book");
        }
#endif
        
        /* find the deleted entry in tableView */
        const int section = indexPath.section;
        id <NSFetchedResultsSectionInfo> sectionInfo = [[fetchedResultsController sections] objectAtIndex:section];
        const int numberOfObjects = [sectionInfo numberOfObjects];
        //NSLog(@"numberOfObjects=%d %d, row=%d, section=%d, name=%@",
        //      numberOfObjects, [sectionInfo.objects count], indexPath.row, indexPath.section, [sectionInfo name]);
        /* delete the object */
        [pContext deleteObject:pSelectedBook];

        /* Commit the change. */
        NSError *error;
        if(![pContext save:&error]) {
#ifdef DEVELOPMENT_ONLY
            // Update to handle the error appropriately.
            NSLog(@"Deleting - Unresolved error %@, %@", error, [error userInfo]);
            exit(-1);  // Fail
#endif
        }

        /* remove section or row in tableView
         * it must find first, before pContext deletes object
         */

        //[self constructTableGroups];
        if(numberOfObjects == 1) {
            [tableView deleteSections:[NSIndexSet indexSetWithIndex:section]
                     withRowAnimation:UITableViewRowAnimationFade];
        } else {
            [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]
                             withRowAnimation:UITableViewRowAnimationFade];
        }
        [tableView endUpdates];
        /*  update flag  */
        mbIsUpdated = YES;
    }
}

#pragma mark -
#pragma mark Selection and moving

//-----------------------------------------------------------------------------
/*! Display the details of a book at a detail view controller.
 */
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
/* Create and push a detail view controller. */
#if 1
    BookDetailViewCtrl *pBookDetailViewCtrl = [[BookDetailViewCtrl alloc] initWithNibName:@"BookDetailView" bundle:nil];
#else
    //BookDetailViewCtrl *pBookDetailViewCtrl = [[BookDetailViewCtrl alloc] init];
    BookDetailViewCtrl *pBookDetailViewCtrl = [[BookDetailViewCtrl alloc] initWithStyle:UITableViewStyleGrouped];
#endif
    pBookDetailViewCtrl.bookDetailViewCtrlDelegate = self;
    Book *pSelectedBook = (Book *)[[self fetchedResultsController] objectAtIndexPath:indexPath];
    // Pass the selected book to the new view controller.
    pBookDetailViewCtrl.book = pSelectedBook;
    [self.navigationController pushViewController:pBookDetailViewCtrl animated:YES];
    [pBookDetailViewCtrl release];
}

//------------------------------------------------------------------------------
/*! Can the table view be re-orderable? 
 */
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    return NO;      /* The table view should not be re-orderable. */
}

//------------------------------------------------------------------------------
/*! Individual rows can opt out of having the -editing property set for them.
 * If not implemented, all rows are assumed to be editable.
 */
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
/* protect the current selected book */
    Book *pSelectedBook = (Book*)[fetchedResultsController objectAtIndexPath:indexPath];
    return ![mpAppSetting isReadBook:pSelectedBook];    /* NOT logic */
    //return ((indexPath.section == 0) && (indexPath.row == 0)) ? NO : YES;
}

//------------------------------------------------------------------------------
- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 50.0;
}
//=============================================================================
// ACTION METHODS
//=============================================================================
//-----------------------------------------------------------------------------
- (IBAction)onSortingKey
{
    if([mpAppSetting.sortingKey intValue] != xpSortingKey.selectedSegmentIndex) {
        mpAppSetting.sortingKey = [NSNumber numberWithInt:xpSortingKey.selectedSegmentIndex];

        self.fetchedResultsController = nil;
        /* Executes the fetch request on the store to get objects.
         * Returns YES if successful or NO (and an error) if a problem occurred. 
         * An error is returned if the fetch request specified doesn't include
         * a sort descriptor that uses sectionNameKeyPath.
         * After executing this method, the fetched objects can be accessed
         * with the property 'fetchedObjects'
         */
        NSError *error;
        if(![[self fetchedResultsController] performFetch:&error]) {
#ifdef DEVELOPMENT_ONLY
            // Update to handle the error appropriately.
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            exit(-1);  // Fail
#endif
        }
        [xpTableView reloadData];

        /* Commit the change. */
        NSManagedObjectContext *pContext = [fetchedResultsController managedObjectContext];
        NSError *pErrorDB;
        if(![pContext save:&pErrorDB]) {
#ifdef DEVELOPMENT_ONLY
            // Update to handle the error appropriately.
            NSLog(@"Sortingkey - Unresolved error %@, %@", pErrorDB, [pErrorDB userInfo]);
            exit(-1);  // Fail
#endif
        }
    }
       
}

//-----------------------------------------------------------------------------
- (IBAction)onDone
{
    [self.delegate bookListViewCtrlDidFinish:self isUpdated:mbIsUpdated];    
}

#pragma mark -
#pragma mark Add a book 

//------------------------------------------------------------------------------
/*!  Creates a new book, an BookAddViewCtrl to manage addition of the book,
 * and a new managed object context for the add controller to keep changes made to
 * the book discrete from the application's managed object context until the book is saved.
 *
 * IMPORTANT: It's not necessary to use a second context for this.
 * You could just use the existing context, which would simplify some of the code
 * -- you wouldn't need to merge changes after a save, for example.
 *  This implementation, though, illustrates a pattern that may sometimes be useful
 *  (where you want to maintain a separate set of edits).
 *
 * The root view controller sets itself as the delegate of the add controller
 * so that it can be informed when the user has completed the add operation
 * -- either saving or canceling (see bookAddViewCtrl:didFinishWithSave:).
 */
- (IBAction)onAddBook
{
/* Create a new managed object context for the new book
 * -- set its persistent store coordinator to the same as
 *  that from the fetched results controller's context.
 */
    NSManagedObjectContext *addingContext = [[NSManagedObjectContext alloc] init];
    self.addingManagedObjectContext = addingContext;
    [addingContext release];

/* the bridge between the control & access layers. */
    NSManagedObjectContext *pContext = [fetchedResultsController managedObjectContext];
    NSPersistentStoreCoordinator *pCoordinator = [pContext persistentStoreCoordinator];
    [addingManagedObjectContext setPersistentStoreCoordinator:pCoordinator];

/* create a view control to add a new book */
#if 1
    BookAddViewCtrl *pBookAddViewCtrl = [[BookAddViewCtrl alloc] initWithNibName:@"BookDetailView" bundle:nil];
#else
    BookAddViewCtrl *pBookAddViewCtrl = [[BookAddViewCtrl alloc] initWithStyle:UITableViewStyleGrouped];
#endif
    pBookAddViewCtrl.bookAddViewCtrlDelegate = self;
    pBookAddViewCtrl.book = (Book *)[NSEntityDescription
                insertNewObjectForEntityForName:@"Book" inManagedObjectContext:addingContext];
    [pBookAddViewCtrl.book defaultSettingWithCategory:eCategory_New appSetting:mpAppSetting];

/* present a NavCtrl */
    UINavigationController *pNavCtrl = [[UINavigationController alloc] initWithRootViewController:pBookAddViewCtrl];
    [pBookAddViewCtrl release];
    [self.navigationController presentModalViewController:pNavCtrl animated:YES];
    [pNavCtrl release];
}

//=============================================================================
// PROTOCOL METHODS
//=============================================================================
- (void)bookDetailViewCtrl:(BookDetailViewCtrl *)pViewCtrl didFinishWithSave:(BOOL)bIsSave
{
/* save, if it is changed */
    if(bIsSave) {
        NSManagedObjectContext *pContext = [fetchedResultsController managedObjectContext];

        NSError *error;
        if(![pContext save:&error]) {
#ifdef DEVELOPMENT_ONLY
            // Update to handle the error appropriately.
            NSLog(@"Updating - Unresolved error %@, %@", error, [error userInfo]);
            exit(-1);  // Fail
#endif
        }
        /* in case category is changed, redo the fetch */
        [self redoPerformFetch];
    }
/* Don't pass current value to the edited object, just pop. */
    [self.navigationController popViewControllerAnimated:YES];
}

//------------------------------------------------------------------------------
/*! Add controller's delegate method;
 * informs the delegate that the add operation has completed,
 * and indicates whether the user saved the new book.
 */
/*
 * The new book is associated with the add controller's managed object context.
 * This is good because it means that any edits that are made
 * don't affect the application's main managed object context
 * -- it's a way of keeping disjoint edits in a separate scratchpad --
 * but it does make it more difficult to get the new book registered with the
 * fetched results controller.
 *
 * First, you have to save the new book.
 * This means it will be added to the persistent store.
 * Then you can retrieve a corresponding managed object into the application delegate's context.
 * Normally you might do this using a fetch or using objectWithID: -- for example
 *
 * NSManagedObjectID *newBookID = [controller.book objectID];
 * NSManagedObject *newBook = [applicationContext objectWithID:newBookID];
 *
 * These techniques, though, won't update the fetch results controller,
 * which only observes change notifications in its context.
 * You don't want to tell the fetch result controller to perform its fetch again
 * because this is an expensive operation.
 * You can, though, update the main context using mergeChangesFromContextDidSaveNotification:
 * which will emit change notifications that the fetch results controller will observe.
 * To do this:
 *    1 Register as an observer of the add controller's change notifications
 *    2 Perform the save
 *    3 In the notification method (addControllerContextDidSave:), merge the changes
 *    4 Unregister as an observer
 */
- (void)bookAddViewCtrl:(BookAddViewCtrl *)pViewCtrl didFinishWithSave:(BOOL)bIsSave
{
    if(bIsSave) {
        NSNotificationCenter *dnc = [NSNotificationCenter defaultCenter];
        [dnc addObserver:self
                selector:@selector(addControllerContextDidSave:)
                    name:NSManagedObjectContextDidSaveNotification
                  object:addingManagedObjectContext];

        NSError *error;
        if(![addingManagedObjectContext save:&error]) {
#ifdef DEVELOPMENT_ONLY
            // Update to handle the error appropriately.
            NSLog(@"Adding - Unresolved error %@, %@", error, [error userInfo]);
            exit(-1);  // Fail
#endif
        }
        [dnc removeObserver:self
                       name:NSManagedObjectContextDidSaveNotification
                     object:addingManagedObjectContext];

        /* in case category is changed, redo the fetch */
        [self redoPerformFetch];
        [mpAppSetting writeToFile];

        /*  update flag */
        mbIsUpdated = YES;
    }
    // Release the adding managed object context.
    self.addingManagedObjectContext = nil;

    // Dismiss the modal view to return to the main list
    [self dismissModalViewControllerAnimated:YES];
}

#if 1
//------------------------------------------------------------------------------
/*! workaround solution for deleting table cells */
- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller {
    if (!self.xpTableView.editing)
        [self.xpTableView reloadData];
}
#endif

//------------------------------------------------------------------------------
/*! Notification from the add controller's context's save operation.
 * This is used to update the fetched results controller's managed object context
 * with the new book instead of performing a fetch
 * (which would be a much more computationally expensive operation).
 */
- (void)addControllerContextDidSave:(NSNotification*)saveNotification
{
    NSManagedObjectContext *pContext = [fetchedResultsController managedObjectContext];
    // Merging changes causes the fetched results controller to update its results
    [pContext mergeChangesFromContextDidSaveNotification:saveNotification];
}


//==============================================================================
// CoreData Fetch
//==============================================================================
#pragma mark -
#pragma mark Fetched results controller (NSFetchedResultsControllerDelegate)

//------------------------------------------------------------------------------
/*! Returns the fetched results controller.
 * Creates and configures the controller if necessary.
 */
- (NSFetchedResultsController *)fetchedResultsController
{
/* Creates and configures the controller if necessary. */
    if(fetchedResultsController != nil) {
        return fetchedResultsController;
    }

/* sortingKey */
    //NSLog(@"%d", [mpAppSetting.sortingKey intValue]);
    NSString *pSortingKeyInString = [mpAppSetting sortingKeyInString];
    BOOL bSortingKeyIsAscending   = [mpAppSetting sortingKeyIsAscending];

/* Create and configure a fetch request with the Book entity. */
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Book"
                                              inManagedObjectContext:managedObjectContext];
    [fetchRequest setEntity:entity];
    [fetchRequest setFetchBatchSize:20];    /* TODO */

/* Create the sort descriptors array to fetchRequest. */
    NSSortDescriptor *categoryDescriptor = [[NSSortDescriptor alloc] initWithKey:@"category" ascending:YES];
    NSSortDescriptor *titleDescriptor = [[NSSortDescriptor alloc] initWithKey:pSortingKeyInString
                                                                    ascending:bSortingKeyIsAscending];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:categoryDescriptor, titleDescriptor, nil];
    [categoryDescriptor release];
    [titleDescriptor release];

    [fetchRequest setSortDescriptors:sortDescriptors];
    [sortDescriptors release];

/* Create and initialize the fetch results controller (an instance of NSFetchedResultsController)
 * fetchRequest       - the fetch request used to get the objects.
 *                      It's expected that the sort descriptor used in
 *                      the request groups the objects into sections.
 * context            - the context that will hold the fetched objects
 * sectionNameKeyPath - keypath on resulting objects that returns the section name.
 *                      This will be used to pre-compute the section information.
 * cacheName          - Pre computed section info is cached to a private directory under this name.
 *                      If there is pre computed section info stored under this name,
 *                      it is checked to see if it matches the fetchRequest.
 *                      If the cached info doesn't match the request,
 *                      it's deleted and recomputed when the fetch happens.
 */
    NSFetchedResultsController *aFetchedResultsController = [[NSFetchedResultsController alloc]
                        initWithFetchRequest:fetchRequest
                        managedObjectContext:managedObjectContext
                          sectionNameKeyPath:@"category"
                                   cacheName:@"Root"];
    self.fetchedResultsController = aFetchedResultsController;
    /* set delegate */
    fetchedResultsController.delegate = self;
    [aFetchedResultsController release];
    [fetchRequest release];

/* return */
    return fetchedResultsController;
}

//------------------------------------------------------------------------------
/*! redo the fetch */
- (void)redoPerformFetch
{
        self.fetchedResultsController = nil;
        /* Executes the fetch request on the store to get objects.
         * Returns YES if successful or NO (and an error) if a problem occurred. 
         * An error is returned if the fetch request specified doesn't include
         * a sort descriptor that uses sectionNameKeyPath.
         * After executing this method, the fetched objects can be accessed
         * with the property 'fetchedObjects'
         */
        NSError *error;
        if(![[self fetchedResultsController] performFetch:&error]) {
#ifdef DEVELOPMENT_ONLY
            // Update to handle the error appropriately.
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            exit(-1);  // Fail
#endif
        }
        [xpTableView reloadData];

#if 0
        /* Commit the change. */
        NSManagedObjectContext *pContext = [fetchedResultsController managedObjectContext];
        NSError *pErrorDB;
        if(![pContext save:&pErrorDB]) {
#ifdef DEVELOPMENT_ONLY
            // Update to handle the error appropriately.
            NSLog(@"Sortingkey - Unresolved error %@, %@", pErrorDB, [pErrorDB userInfo]);
            exit(-1);  // Fail
#endif
        }
#endif
}

#if 0   /* hopeless (bug in IPHONE_3_1) */
//------------------------------------------------------------------------------
/*! Notifies the delegate that section and object changes (additions, removal, etc)
 * are about to be processed and notifications will be sent. 
 * 
 * Clients utilizing a UITableView may prepare for a batch of updates
 * by responding to this method with -beginUpdates
 *
 * An optional method.
 */
- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller
{
/* The fetch controller is about to start sending change notifications,
 * so prepare the table view for updates.
 */
#ifdef __IPHONE_3_1
    sectionInsertCount = 0;
#endif  /* __IPHONE_3_1 */
    [self.xpTableView beginUpdates];
}

//------------------------------------------------------------------------------
/*! Notifies the delegate that all section and object changes have been sent. 
 *
 * An optional method.
 */
- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
{
/* The fetch controller has sent all current change notifications,
 * so tell the table view to process all updates.
 */
    [self.xpTableView endUpdates];
}

//------------------------------------------------------------------------------
/*! Notifies the delegate that a fetched object has been changed due to an add, remove, move, or update.
 * controller   - controller instance that noticed the change on its fetched objects
 * anObject     - changed object
 * indexPath    - indexPath of changed object (nil for inserts)
 * type         - indicates if the change was an insert, delete, move, or update
 * newIndexPath - the destination path for inserted or moved objects, nil otherwise
 *
 * Changes are reported with the following heuristics:
 * - On Adds and Removes, only the Added/Removed object is reported.
 *   It's assumed that all objects that come after the affected object are also moved,
 *   but these moves are not reported. 
 * - The Move object is reported when the changed attribute on the object is
 *   one of the sort descriptors used in the fetch request.
 *   An update of the object is assumed in this case, but no separate update message is sent to the delegate.
 * - The Update object is reported when an object's state changes,
 *   and the changed attributes aren't part of the sort keys. 
 *
 * An optional method.
 */
- (void)controller:(NSFetchedResultsController *)controller
                        didChangeObject:(id)anObject
                            atIndexPath:(NSIndexPath *)indexPath
                          forChangeType:(NSFetchedResultsChangeType)type
                           newIndexPath:(NSIndexPath *)newIndexPath
{
    UITableView *tableView = self.xpTableView;

    NSLog(@"%d    indexPath section=%d, row=%d",    indexPath!=nil, [indexPath section], [indexPath row]);
    NSLog(@"%d newIndexPath section=%d, row=%d", newIndexPath!=nil, [newIndexPath section], [newIndexPath row]);
    
    switch(type) {
    case NSFetchedResultsChangeInsert:
        NSLog(@"NSFetchedResultsChangeInsert");
        [tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath]
                         withRowAnimation:UITableViewRowAnimationFade];
        break;

    case NSFetchedResultsChangeDelete:
        NSLog(@"NSFetchedResultsChangeDelete");
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]
                         withRowAnimation:UITableViewRowAnimationFade];
        break;

#ifdef __IPHONE_3_1
    case NSFetchedResultsChangeUpdate: {
        NSLog(@"NSFetchedResultsChangeUpdate");
        NSString *sectionKeyPath = [controller sectionNameKeyPath];
        NSLog(@"sectionKeyPath=%@", sectionKeyPath);
        if (sectionKeyPath == nil)
            break;
        NSManagedObject *changedObject = [controller objectAtIndexPath:indexPath];
        NSArray *keyParts = [sectionKeyPath componentsSeparatedByString:@"."];
    	NSLog(@"number of keyParts=%d", [keyParts count]);
        id currentKeyValue = [changedObject valueForKeyPath:sectionKeyPath];
        for (int i = 0; i < [keyParts count] - 1; i++) {
            NSString *onePart = [keyParts objectAtIndex:i];
            changedObject = [changedObject valueForKey:onePart];
        }
        sectionKeyPath = [keyParts lastObject];
        NSDictionary *committedValues = [changedObject committedValuesForKeys:nil];
        
        NSLog(@"currentKeyValue=%@, committed=%@, sectionKeyPath=%@", currentKeyValue, [committedValues valueForKeyPath:sectionKeyPath], sectionKeyPath);
        NSLog(@"newIndexPath %d section=%d", (newIndexPath != nil), [newIndexPath section]);
        if ([[committedValues valueForKeyPath:sectionKeyPath] isEqual:currentKeyValue])
            break;
            
        NSUInteger tableSectionCount = [tableView numberOfSections];
        NSUInteger frcSectionCount = [[controller sections] count];
        NSLog(@"tableSectionCount=%d, frcSectionCount=%d, sectionInserCount=%d",
              tableSectionCount, frcSectionCount, sectionInsertCount);
        if (tableSectionCount + sectionInsertCount != frcSectionCount) {
            // Need to insert a section
            NSArray *sections = controller.sections;
            NSInteger newSectionLocation = -1;
            for (id oneSection in sections) {
                NSString *sectionName = [oneSection name];
                if ([currentKeyValue isEqual:sectionName]) {
                    newSectionLocation = [sections indexOfObject:oneSection];
                    break;
                }
            }
            if (newSectionLocation == -1)
                return; // uh oh
            
            if (!((newSectionLocation == 0) && (tableSectionCount == 1) && ([tableView numberOfRowsInSection:0] == 0))) {
                [tableView insertSections:[NSIndexSet indexSetWithIndex:newSectionLocation] withRowAnimation:UITableViewRowAnimationFade];
                sectionInsertCount++;
            }
            
            NSUInteger indices[2] = {newSectionLocation, 0};
            newIndexPath = [[[NSIndexPath alloc] initWithIndexes:indices length:2] autorelease];
        }
        //break;    /* TODO: it does not break here? */
    }
    case NSFetchedResultsChangeMove:
        NSLog(@"NSFetchedResultsChangeMove");
        if (newIndexPath != nil) {
            NSUInteger tableSectionCount = [tableView numberOfSections];
            NSUInteger frcSectionCount = [[controller sections] count];
            NSLog(@"tableSectionCount=%d, frcSectionCount=%d, sectionInserCount=%d",
                  tableSectionCount, frcSectionCount, sectionInsertCount);
            if (frcSectionCount != tableSectionCount + sectionInsertCount)  {
                [tableView insertSections:[NSIndexSet indexSetWithIndex:[newIndexPath section]] withRowAnimation:UITableViewRowAnimationNone];
                sectionInsertCount++;
            }
                
            [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]
                             withRowAnimation:UITableViewRowAnimationFade];
            [tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath]
                             withRowAnimation:UITableViewRowAnimationRight];
            
        } else {
            NSLog(@"indexPath section=%d", [indexPath section]);
            [tableView reloadSections:[NSIndexSet indexSetWithIndex:[indexPath section]] withRowAnimation:UITableViewRowAnimationFade];
        }
        break;
#else
    case NSFetchedResultsChangeUpdate:
#if 0
        /* it is not necessary to configure cells for ChangeUpdate
         * The update is triggered from BookDetailViewCtrl in this project.
         */
        [self configureCell:[tableView cellForRowAtIndexPath:indexPath]
                atIndexPath:indexPath];
#endif
        break;

    case NSFetchedResultsChangeMove: /* TODO: it might be buggy */
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]
                         withRowAnimation:UITableViewRowAnimationFade];
        // Reloading the section inserts a new row and ensures that titles are updated appropriately.
        [tableView reloadSections:[NSIndexSet indexSetWithIndex:newIndexPath.section]
                 withRowAnimation:UITableViewRowAnimationFade];
        break;
#endif  /* __IPHONE_3_1 */
    default:
        break;
    }
}

/*! Notifies the delegate of added or removed sections. 
 * controller  - controller instance that noticed the change on its sections
 * sectionInfo - changed section
 * index       - index of changed section
 * type        - indicates if the change was an insert or delete
 *
 * Changes on section info are reported before changes on fetchedObjects. 
 *
 * An optional method.
 */
- (void)controller:(NSFetchedResultsController *)controller
                       didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo
                                atIndex:(NSUInteger)sectionIndex
                          forChangeType:(NSFetchedResultsChangeType)type
{
    UITableView *tableView = self.xpTableView;

    NSLog(@"sectionIndex=%d, sectionInsertCount=%d, numSections=%d, type=%d",
          sectionIndex, sectionInsertCount, [tableView numberOfSections], type);
#ifdef __IPHONE_3_1
    switch(type) {
    case NSFetchedResultsChangeInsert:
        NSLog(@"NSFetchedResultsChangeInsert");
        //if (!( (sectionIndex == 0) && ([tableView numberOfSections] == 1) && ([tableView numberOfRowsInSection:0] == 0) )) {
        //if (!( (sectionIndex == 0) && ([tableView numberOfSections] == 1) ))
        {
            [tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex]
                     withRowAnimation:UITableViewRowAnimationFade];
            sectionInsertCount++;
        }
            
        break;
    case NSFetchedResultsChangeDelete:
        NSLog(@"NSFetchedResultsChangeDelete");
        if (!((sectionIndex == 0) && ([tableView numberOfSections] == 1) )) {
            [tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex]
                     withRowAnimation:UITableViewRowAnimationFade];
            sectionInsertCount--;
        }
        break;
    case NSFetchedResultsChangeMove:
        NSLog(@"NSFetchedResultsChangeMove");
        break;
    case NSFetchedResultsChangeUpdate:
        NSLog(@"NSFetchedResultsChangeUpdate");
        break;
    default:
        break;
    }
	NSLog(@"sectionIndex=%d, sectionInsertCount=%d", sectionIndex, sectionInsertCount);
#else
    switch(type) {
    case NSFetchedResultsChangeInsert:
        [tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex]
                      withRowAnimation:UITableViewRowAnimationFade];
        break;

    case NSFetchedResultsChangeDelete:
        [tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex]
                      withRowAnimation:UITableViewRowAnimationFade];
        break;
    }
#endif  /* __IPHONE_3_1 */
}
#endif  /* end of hopeless */

//=============================================================================
@end
