/*!
 * \file EditingViewCtrl.m
 *      The table view controller responsible for editing a field of data, text or date.
 */
/***************************************************************************/
/* Copyright (c) 2010 AuroraWand LLC                                       */
/* All rights reserved.                                                    */
/*                                                                         */
/* No part of this document may be copied, reproduced,                     */
/* or transmitted in any form without the prior written                    */
/* consent of AuroraWand LLC, San Jose, CA                                 */
/*                                                                         */
/* Created by LinCheng Wang on 05/26/10                                    */
/***************************************************************************/

//=============================================================================
// COMPILATION OPTIONS
//=============================================================================
//#define DEBUG_LEVEL     /* to turn on message utility by UtilMsg.h */

//=============================================================================
// HEADER FILES
//=============================================================================
#import "EditingViewCtrl.h"

//=============================================================================
// MACROS & INLINE FUNCTIONS
//=============================================================================

//=============================================================================
// TYPEDEFS & DATA STRUCTURES
//=============================================================================

//=============================================================================
// CLASS IMPLEMENTATION 
//=============================================================================
@implementation EditingViewCtrl

@synthesize delegate;
@synthesize xpTextField, xpNumberField, xpDatePicker;
@synthesize editedObject, editedFieldId, editedFieldKey, editedFieldName;
@synthesize meDataType, mnNumberMax, mnNumberMin;

//=============================================================================
// METHODS
//=============================================================================
#pragma mark -
#pragma mark View lifecycle

//-----------------------------------------------------------------------------
- (void)viewDidLoad {
	// Set the title to the user-visible name of the field.
	self.title = editedFieldName;

	// Configure the save and cancel buttons.
	UIBarButtonItem *saveButton = [[UIBarButtonItem alloc]
        initWithBarButtonSystemItem:UIBarButtonSystemItemSave
                             target:self
                             action:@selector(onSave)];
	self.navigationItem.rightBarButtonItem = saveButton;
	[saveButton release];
	
	UIBarButtonItem *cancelButton = [[UIBarButtonItem alloc]
        initWithBarButtonSystemItem:UIBarButtonSystemItemCancel
                             target:self
                             action:@selector(onCancel)];
	self.navigationItem.leftBarButtonItem = cancelButton;
	[cancelButton release];
}

//-----------------------------------------------------------------------------
- (void)viewWillAppear:(BOOL)animated {
	
	[super viewWillAppear:animated];
	
// Configure the user interface according to state.
    xpTextField.hidden   = YES;
    xpNumberField.hidden = YES;
    xpDatePicker.hidden  = YES;

    switch(meDataType) {
    case eDataType_Date:
        xpDatePicker.hidden  = NO;
		NSDate *date = [editedObject valueForKey:editedFieldKey];
        if (date == nil) date = [NSDate date];
        xpDatePicker.date = date;
        break;
    case eDataType_Text:
        xpTextField.hidden   = NO;
        xpTextField.text = [editedObject valueForKey:editedFieldKey];
		xpTextField.placeholder = self.title;
        [xpTextField becomeFirstResponder];
        break;
    case eDataType_Number:
        {
            xpNumberField.hidden = NO;
            NSNumber *pNumber = [editedObject valueForKey:editedFieldKey];
            xpNumberField.text = [NSString stringWithFormat:@"%d", [pNumber intValue]];
            xpNumberField.placeholder = self.title;
            [xpNumberField becomeFirstResponder];
            break;
        }
    }
}


#pragma mark -
#pragma mark Memory management

//-----------------------------------------------------------------------------
- (void)dealloc {
    [xpTextField release];
    [xpDatePicker release];

    [editedObject release];
    [editedFieldKey release];
    [editedFieldName release];
	[super dealloc];
}

//=============================================================================
// ACTION METHODS
//=============================================================================
#pragma mark -
#pragma mark Save and cancel operations

//-----------------------------------------------------------------------------
- (IBAction)onSave {
    
// Set the action name for the undo operation.
	NSUndoManager * undoManager = [[editedObject managedObjectContext] undoManager];
	[undoManager setActionName:[NSString stringWithFormat:@"%@", editedFieldName]];
	
// Pass current value to the edited object, then pop.
    switch(meDataType) {
    case eDataType_Date:
        [editedObject setValue:xpDatePicker.date forKey:editedFieldKey];
        break;
    case eDataType_Text:
        [editedObject setValue:xpTextField.text forKey:editedFieldKey];
        break;
    case eDataType_Number:
        {
            int nVal = [xpNumberField.text intValue];
            if(nVal > mnNumberMax)
                nVal = mnNumberMax;
            else if(nVal < mnNumberMin)
                nVal = mnNumberMin;
            NSNumber *pNumber = [NSNumber numberWithInt:nVal];
            [editedObject setValue:pNumber forKey:editedFieldKey];
            break;
        }
    }
	
/* inform the save to delegate */
    [self.delegate editingViewCtrlDidSave:self];

/* pop back to previous view controller */
    [self.navigationController popViewControllerAnimated:YES];
}

//-----------------------------------------------------------------------------
- (IBAction)onCancel {
    // Don't pass current value to the edited object, just pop.
    [self.navigationController popViewControllerAnimated:YES];
}

//=============================================================================
@end    /* EditingViewCtrl */
