/*!
 * \file  MainViewCtrl.m
 *       Implement MainView controller for BuddhaCounter project
 */
/***************************************************************************/
/* Copyright (c) 2010 AuroraWand LLC                                       */
/* All rights reserved.                                                    */
/*                                                                         */
/* No part of this document may be copied, reproduced,                     */
/* or transmitted in any form without the prior written                    */
/* consent of AuroraWand LLC, San Jose, CA                                 */
/*                                                                         */
/* Created by LinCheng Wang on 05/25/10                                    */
/***************************************************************************/

//=============================================================================
// COMPILATION OPTIONS
//=============================================================================
//#define DEBUG_LEVEL     /* to turn on message utility by UtilMsg.h */

//=============================================================================
// HEADER FILES
//=============================================================================
#import "AudioPlayerUtil.h"
#import <AudioToolbox/AudioToolbox.h>

#import "AppDefine.h"
#import "MainViewCtrl.h"
#import "AppModel.h"
#import "AppSetting.h"

//=============================================================================
// MACROS & INLINE FUNCTIONS
//=============================================================================
//#define kCounterNumComp     6   /* defined in h file */
#define kCounterMultiple             3
#define kCounterMultipleInHalf      (kCounterMultiple / 2)
#define kCounterNumRow              10
#define kCounterNumRowInView        (kCounterNumRow * kCounterMultiple)
#define kCounterNumRowInViewInHalf  (kCounterNumRow * kCounterMultipleInHalf)

//=============================================================================
// TYPEDEFS and DATA STRUCTURES
//=============================================================================

//=============================================================================
// DECLARATION PRIVATE METHODS
//=============================================================================
@interface MainViewCtrl (Private)
- (void)displayTargetCountOfBook;
- (void)setCountVal:(int)nCountDelta isReset:(BOOL)bIsReset;
- (void)setPickerCounterVal:(int)nCountDelta isReset:(BOOL)bIsReset;
- (void)showBtnCountDone:(BOOL)bIsDone;
- (void)updateReadBook:(Book*)pReadBook;
- (void)enableCountReset:(BOOL)bReset withRestore:(BOOL)bRestore;
- (void)newDatabase;

- (void)updateFetchedResultsController;
@end

//=============================================================================
// CLASS IMPLEMENTATION
//=============================================================================
@implementation MainViewCtrl

/* accessors */
@synthesize xpPickerBook, xpPickerCounter;
@synthesize xpBtnCountUp, xpBtnCountDown, xpBtnCountDone, xpBtnCountReset, xpBtnCountRestore,
            xpBtnShowInfo, xpBtnShowOption, xpBtnBell;
@synthesize xpLblCountTarget, xpLblCountTotal, xpPbarCountNow;
@synthesize xpImageViewCoverDone;

@synthesize fetchedResultsController, managedObjectContext;
@synthesize mpAppSetting;
@synthesize mpReadBook;

@synthesize mpSoundReminder, mpSoundComplete;

//=============================================================================
// METHODS
//=============================================================================

//-----------------------------------------------------------------------------
/*! Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
 */
- (void)viewDidLoad
{
/* load super first */
    [super viewDidLoad];

/* User Interface, hacked for XIB for initialization */
    xpPickerCounter.backgroundColor = [UIColor blackColor]; /* picker color */

    [self enableCountReset:YES withRestore:NO];
    const BOOL bPlaySound = [mpAppSetting.bPlaySound intValue];
    xpBtnBell.selected = bPlaySound;
    xpImageViewCoverDone.hidden = YES;

/* take care of the plus/minus/done buttons */
    xpBtnCountDone.alpha = 0.0;                             /* reset CountDone */
    const BOOL bCountingOrder = [mpAppSetting.bCountingOrder intValue];
    /* make the inverse logic for routine to correct and trigger */
    mpBtnCountPtr = (bCountingOrder == kCountingOrderUp) ? xpBtnCountDown : xpBtnCountUp;

/* Executes the fetch request on the store to get objects.
 * Returns YES if successful or NO (and an error) if a problem occurred. 
 * An error is returned if the fetch request specified doesn't include
 * a sort descriptor that uses sectionNameKeyPath.
 * After executing this method, the fetched objects can be accessed
 * with the property 'fetchedObjects'
 */
    NSError *error;
    if(![[self fetchedResultsController] performFetch:&error]) {
#ifdef DEVELOPMENT_ONLY
        // Update to handle the error appropriately.
        NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        exit(-1);  // Fail
#endif
    }

#if 1
    const int nCount = [[fetchedResultsController fetchedObjects] count];
    if(nCount == 0) {
        [self newDatabase];
        /* it is a bug fix for SDK 3.2 and 4.0, which are running at Xcode 3.2.3.
         * The bug is that it crashes at the first run.
         */
        [self updateFetchedResultsController];
    } else {
        //NSArray *pBookArray = [fetchedResultsController fetchedObjects];
        //mpAppSetting.selectedBook = [pBookArray objectAtIndex:0];
    }

#endif
/* Images and ImageViews for counter Picker */
    mpImageArray = [[NSMutableArray alloc] init];
    for(int nRow = 0; nRow < kCounterNumRow; nRow++) {
        NSString *psImageName = [NSString stringWithFormat:@"%d.png", nRow];
        UIImage *pImage = [UIImage imageNamed:psImageName];
        [mpImageArray insertObject:pImage atIndex:nRow];
    }

/* sounds */
    AVAudioPlayer *pSoundReminder = [AVAudioPlayer createAudioPlayerWithPath:@"WoodFish" ofType:@"caf"]; 
    [pSoundReminder prepareToPlay]; /* preload sound file */
    self.mpSoundReminder = pSoundReminder;
    [pSoundReminder release];
    AVAudioPlayer *pSoundComplete = [AVAudioPlayer createAudioPlayerWithPath:@"Bowl"     ofType:@"mp3"]; 
    self.mpSoundComplete = pSoundComplete;
    [pSoundComplete release];

}

//-----------------------------------------------------------------------------
- (void)viewWillAppear:(BOOL)animated
{

/* Redisplay the data. */
    [self.xpPickerBook reloadAllComponents];
}

//-----------------------------------------------------------------------------
- (void)viewDidAppear:(BOOL)animated
{
/* search the selected book and set
 * It is moved to viewWillAppear due to
 * 1. the change of SortingKey in OptionView changes the order of fetchedObjects.
 * 2. BookListView may delete/increate/move books.
 */
/* the IdKey is not associated with the location (nRow) in picker */
    NSArray *pBookArray = [fetchedResultsController fetchedObjects];
    for(int nRow = 0; nRow < [pBookArray count]; nRow++) {
        Book *pBook = (Book*)[pBookArray objectAtIndex:nRow];
        if([mpAppSetting isReadBook:pBook]) {
            [xpPickerBook selectRow:nRow inComponent:0 animated:YES];
            mpReadBook = pBook; /* TODO */
            break;
        }
    }

/* TODO: what happens if not found? not possible */
    if(mpReadBook == nil) {
        //NSLog(@"not found");
        mpReadBook = (Book*)[pBookArray objectAtIndex:0];
        [mpAppSetting setReadBook:mpReadBook];
    }
    
/* set target count */
    [self displayTargetCountOfBook];  /* TODO */
}

//-----------------------------------------------------------------------------
- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];

    // Release any cached data, images, etc. that aren't in use.
}

//-----------------------------------------------------------------------------
/*! Release any retained subviews of the main view. */
- (void)viewDidUnload
{
/* Release any properties that are loaded in viewDidLoad or can be recreated lazily. */
/* view objects */
    self.xpPickerBook      = nil;
    self.xpPickerCounter   = nil;
    self.xpBtnCountUp      = nil;
    self.xpBtnCountDown    = nil;
    self.xpBtnCountDone    = nil;
    self.xpBtnCountReset   = nil;
    self.xpBtnCountRestore = nil;
    self.xpBtnShowInfo     = nil;
    self.xpBtnShowOption   = nil;
    self.xpBtnBell         = nil;
    self.xpLblCountTarget  = nil;
    self.xpLblCountTotal   = nil;
    self.xpPbarCountNow    = nil;

    self.xpImageViewCoverDone = nil;

/* sounds */
    self.mpSoundReminder = nil;
    self.mpSoundComplete = nil;

/* data Model for PickerBook */
    self.fetchedResultsController = nil;

/* release super at last */
    [super viewDidUnload];
}

//-----------------------------------------------------------------------------
/*! Called when the view is dismissed, covered or otherwise hidden.
 * Default does nothing
 */
- (void)viewWillDisappear:(BOOL)animated
{
/* super first */
    [super viewWillDisappear:animated];
    
/* set to AppSetting */
    [self updateReadBook:nil];
}

#if 0
//-----------------------------------------------------------------------------
/*! Called after the view was dismissed, covered or otherwise hidden.
 * Default does nothing
 */
- (void)viewDidDisappear:(BOOL)animated
{
/* super first */
    [super viewDidDisappear:animated];
}
#endif

/*
   //-----------------------------------------------------------------------------
   // Override to allow orientations other than the default portrait orientation.
   - (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations.
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
   }
 */

//-----------------------------------------------------------------------------
/*! Release objects */
- (void)dealloc
{
/* view objects */
    [xpPickerBook         release];
    [xpPickerCounter      release];
    [xpBtnCountUp         release];
    [xpBtnCountDown       release];
    [xpBtnCountDone       release];
    [xpBtnCountReset      release];
    [xpBtnCountRestore    release];
    [xpBtnShowInfo        release];
    [xpBtnShowOption      release];
    [xpBtnBell            release];
    [xpLblCountTarget     release];
    [xpLblCountTotal      release];
    [xpPbarCountNow       release];

    [xpImageViewCoverDone release];

/* sounds */
    [mpSoundReminder      release];
    [mpSoundComplete      release];

/* data Model for PickerBook */
    [fetchedResultsController release];
    [managedObjectContext     release];

/* AppSetting */
    [mpAppSetting             release];

/* picker data */
    [mpImageArray removeAllObjects];
    [mpImageArray release];

/* release super at last */
    [super dealloc];
}

//=============================================================================
// PRIVATE METHODS
//=============================================================================
//-----------------------------------------------------------------------------
- (void)displayTargetCountOfBook
{
/* Load AppSetting */
    mnCountNow    = [mpReadBook.countNow    intValue];
    mnCountTarget = [mpReadBook.countTarget intValue];
    mnCountTotal  = [mpReadBook.countTotal  intValue];
    
/* whether this book has been done reading */
    [self showBtnCountDone:(mnCountNow >= mnCountTarget)];

/* update bell */
    const BOOL bPlaySound = [mpAppSetting.bPlaySound intValue];
    xpBtnBell.selected = bPlaySound;

/* set picker value */
    [self setCountVal:mnCountNow isReset:YES];

/* target label in progress bar */
    xpLblCountTarget.text = [NSString stringWithFormat:@"%d", mnCountTarget];

/* plus or minus image */
    const BOOL bCountingOrder = [mpAppSetting.bCountingOrder intValue];
    //NSLog(@"CountingOrder=%d, Done.Enabled=%d, (DOWN) %4.1f %d (UP) %4.1f %d",
    //        bCountingOrder, xpBtnCountDone.enabled,
    //        xpBtnCountDown.alpha, xpBtnCountDown.enabled,
    //        xpBtnCountUp.alpha, xpBtnCountUp.enabled);
    if(bCountingOrder == kCountingOrderUp) {
        if(mpBtnCountPtr != xpBtnCountUp) {
            xpBtnCountUp.enabled   = !(xpBtnCountDone.enabled);
            xpBtnCountUp.alpha     = (xpBtnCountUp.enabled) ? 1.0 : 0.0;
            xpBtnCountDown.enabled = NO;
            xpBtnCountDown.alpha   = 0.0;
            mpBtnCountPtr = xpBtnCountUp;
        }
    } else {
        if(mpBtnCountPtr != xpBtnCountDown) {
            xpBtnCountUp.enabled   = NO;
            xpBtnCountUp.alpha     = 0.0;
            xpBtnCountDown.enabled = !(xpBtnCountDone.enabled);
            xpBtnCountDown.alpha   = (xpBtnCountDown.enabled) ? 1.0 : 0.0;
            mpBtnCountPtr = xpBtnCountDown; 
        }
    }
    //NSLog(@"CountingOrder=%d, Done.Enabled=%d, (down) %4.1f %d (up) %4.1f %d",
    //        bCountingOrder, xpBtnCountDone.enabled,
    //        xpBtnCountDown.alpha, xpBtnCountDown.enabled,
    //        xpBtnCountUp.alpha, xpBtnCountUp.enabled);
}

//-----------------------------------------------------------------------------
/*!
 * nCountDelta is always positive.
 */
- (void)setCountVal:(int)nCountDelta isReset:(BOOL)bIsReset
{
/* set current count */
    if(bIsReset) {
        mnCountNow = nCountDelta;
    } else {
        mnCountNow   += nCountDelta;
        mnCountTotal += nCountDelta;
    }
/* update countTotal label - not necessarily updated for all cases */
    xpLblCountTotal.text = [NSString stringWithFormat:@"%d", mnCountTotal];
    
/* set counter picker */
    BOOL bCountingOrder = [mpAppSetting.bCountingOrder intValue];
    int nCountPicker = (bCountingOrder == kCountingOrderUp) ? mnCountNow : (mnCountTarget - mnCountNow);
    if(bIsReset) {
        if(nCountPicker < 0)
            nCountPicker = 0;
        [self setPickerCounterVal:nCountPicker isReset:bIsReset];
    } else {
        if(bCountingOrder == kCountingOrderDown)
            nCountDelta = -nCountDelta;
        [self setPickerCounterVal:nCountDelta isReset:bIsReset];
    }

/* set progress bar */
    //NSLog(@"target=%d, now=%d", mnCountTarget, mnCountNow);
    float fProgress = 0.001;
    if(mnCountTarget != 0) {
        fProgress = ((float) (mnCountNow ? mnCountNow : 1)) / mnCountTarget;
    }
    [xpPbarCountNow setProgress:fProgress];
}

//-----------------------------------------------------------------------------
- (void)setPickerCounterVal:(int)nCountDelta isReset:(BOOL)bIsReset
{
    static int snCountPicker = 0;
    if(bIsReset) {
        snCountPicker = nCountDelta;
        for(int nComp = kCounterNumComp - 1; nComp >= 0; nComp--) {
            const int nDigit = nCountDelta % kCounterNumRow;
            nCountDelta /= kCounterNumRow;
            const int nRow   = nDigit + kCounterNumRowInViewInHalf;
            manPickerCounterRow[nComp] = nRow;
            [xpPickerCounter selectRow:nRow inComponent:nComp animated:YES];
            //CGSize aSize = [xpPickerCounter rowSizeForComponent:nComp];
            //NSLog(@"%d: %8.3f %8.3f", nComp, aSize.width, aSize.height);
        }
    } else {
        snCountPicker += nCountDelta;
        if(snCountPicker < 0) {
            snCountPicker = 0;
        } else {
            int nComp;
            for(nComp = kCounterNumComp - 1; nComp >= 0; nComp--) {
                int nRow = manPickerCounterRow[nComp] + nCountDelta;
                manPickerCounterRow[nComp] = nRow;
                //NSLog(@"> nComp=%d, nRow=%d", nComp, manPickerCounterRow[nComp]);
                if(nRow < kCounterNumRowInViewInHalf) { /* count down borrow */
                    manPickerCounterRow[nComp] += kCounterNumRow;   /* wrap */
                    [xpPickerCounter selectRow:manPickerCounterRow[nComp] - nCountDelta inComponent:nComp animated:NO];
                    nCountDelta = -1; /* borrow */
                } else if(nRow >= kCounterNumRowInViewInHalf + kCounterNumRow) { /* count up carry */
                    manPickerCounterRow[nComp] -= kCounterNumRow;   /* wrap */
                    [xpPickerCounter selectRow:manPickerCounterRow[nComp] - nCountDelta inComponent:nComp animated:NO];
                    nCountDelta =  1; /* carry */
                } else {    /* neither carry nor borrow -> stop updating */
                    [xpPickerCounter selectRow:nRow inComponent:nComp animated:YES];
                    break;
                }
                //NSLog(@">>>> nComp=%d, nRow=%d", nComp, manPickerCounterRow[nComp]);
            }

            for(int nCompLast = kCounterNumComp - 1; nCompLast >= nComp + 1; nCompLast--) {
                //NSLog(@">>> nComp=%d, nRow=%d", nCompLast, manPickerCounterRow[nCompLast]);
                [xpPickerCounter selectRow:manPickerCounterRow[nCompLast]     inComponent:nCompLast animated:YES];
                //[xpPickerCounter reloadComponent:nCompLast];
            }
        }
    }
}

//-----------------------------------------------------------------------------
- (void)bellRingOff
{
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDelegate:self];
    [UIView setAnimationDuration:0.5];
    CGAffineTransform transform = CGAffineTransformMakeScale(1.0, 1.0);
    xpBtnBell.imageView.transform = transform;
    xpBtnBell.imageView.alpha = 1.0;
    [UIView commitAnimations];
}

- (void)bellRingOn
{
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDelegate:self];
    [UIView setAnimationDuration:1.0];
    [UIView setAnimationDidStopSelector:@selector(bellRingOff)];
    CGAffineTransform transform = CGAffineTransformMakeScale(2.0, 2.0);
    xpBtnBell.imageView.transform = transform;
    xpBtnBell.imageView.alpha = 0.1;
    [UIView commitAnimations];
}

//-----------------------------------------------------------------------------
- (void)playSound
{
/* return and do nothing, if mnCountTarget is 0 (should not happen) */
    if(mnCountTarget == 0) {
        return;
    }

/* play reminder sound */
    const BOOL bPlaySound = [mpAppSetting.bPlaySound intValue];
    if(bPlaySound) {
        const int nCountSession = [mpAppSetting.countSession intValue];
        if((mnCountNow != 0) && (mnCountNow % nCountSession == 0)) {
            [mpSoundReminder play];
            xpBtnBell.selected = bPlaySound;
            //[self bellRingOn];
        }
    }

/* set button and play complete sound, if complete */
    if(mnCountNow >= mnCountTarget) {
        if(mpBtnCountPtr.enabled == YES) {
            /* play sound */
            if(bPlaySound)
                [mpSoundComplete play];
            else
                AudioServicesPlaySystemSound(kSystemSoundID_Vibrate);
        }
        [self showBtnCountDone:YES];
    }
}

//-----------------------------------------------------------------------------
- (void)showBtnCountDone:(BOOL)bIsDone
{
    if(bIsDone) {
        if(mpBtnCountPtr.enabled == YES) {
            mpBtnCountPtr.enabled  = NO;
            xpBtnCountDone.enabled = YES;
            /* it is a Done */
            [UIView beginAnimations:nil context:nil];
            [UIView setAnimationDelegate:self];
            [UIView setAnimationDuration:2.0];
            mpBtnCountPtr.alpha    = 0.0;
            xpBtnCountDone.alpha   = 1.0;
            [UIView commitAnimations];
            xpImageViewCoverDone.hidden = NO;
        }
    } else {
        if(mpBtnCountPtr.enabled == NO) {
            mpBtnCountPtr.enabled  = YES;
            xpBtnCountDone.enabled = NO;

            [UIView beginAnimations:nil context:nil];
            [UIView setAnimationDelegate:self];
            [UIView setAnimationDuration:1.0];
            mpBtnCountPtr.alpha  = 1.0;
            xpBtnCountDone.alpha = 0.0;
            [UIView commitAnimations];
            xpImageViewCoverDone.hidden = YES;
        }
    }
}

//-----------------------------------------------------------------------------
- (void)updateReadBook:(Book*)pReadBook
{
/* update the current read book first */
    const int nCountOld = [mpReadBook.countNow intValue];
    if(mnCountNow != nCountOld) {   /* updates has been done */
        mpReadBook.countNow        = [NSNumber numberWithInt:mnCountNow];
        mpReadBook.percentComplete = [NSNumber numberWithFloat:(mnCountNow * 100.0) /mnCountTarget];
        mpReadBook.dateLastRead    = [NSDate date];
        //[xpPickerBook reloadAllComponents];
    }

/* update the total count */
    const int nCountTotalOld = [mpReadBook.countTotal intValue];
    if(mnCountTotal != nCountTotalOld) {   /* updates has been done */
        mpReadBook.countTotal      = [NSNumber numberWithInt:mnCountTotal];
    }

/* set the newly read book */
    if(pReadBook) {
        mpReadBook = pReadBook;
        [mpAppSetting setReadBook:pReadBook];
    }

#if 1   /* TODO: save it or not? */
/* save database */
    NSError *pErrorDB = nil;
    if((managedObjectContext != nil) && [managedObjectContext hasChanges]) {
        if(![managedObjectContext save:&pErrorDB]) {
#ifdef DEVELOPMENT_ONLY
			/* Replace this implementation with code to handle the error appropriately.  */
			NSLog(@"Counting - Unresolved error %@, %@", pErrorDB, [pErrorDB userInfo]);
            abort();
#endif
        } 
    }
#endif
}

//-----------------------------------------------------------------------------
- (void)enableCountReset:(BOOL)bReset withRestore:(BOOL)bRestore
{
    static int snCountNow = 0;
    if(bReset) {
        xpBtnCountReset.enabled   = YES;
        xpBtnCountRestore.enabled = NO;
        //if(xpBtnCountReset.enabled == NO) {
        //    xpBtnCountReset.enabled   = YES;
        //    xpBtnCountRestore.enabled = NO;
        //}
        if(bRestore) {
            [self setCountVal:snCountNow isReset:YES];
            [self showBtnCountDone:(mnCountNow >= mnCountTarget)];
        }
    } else {
        if(xpBtnCountReset.enabled == YES) {
            snCountNow = mnCountNow;
            /* disable Reset button until another button
             * enable Restore button
             */ 
            xpBtnCountReset.enabled   = NO;
            xpBtnCountRestore.enabled = YES;
        }
    }
}

//------------------------------------------------------------------------------
- (void)newDatabase
{
#if 1
    NSString *psXmlFile   = [[NSBundle mainBundle] pathForResource:@"NewBooks" ofType:@"plist"];
    NSArray *pBookInfoList = [NSArray arrayWithContentsOfFile:psXmlFile];

    //NSLog(@"count=%d", [pBookInfoList count]);
    int nCount = 0;
    for(NSArray *pBookInfo in pBookInfoList) {
        Book *pBook = (Book *)[NSEntityDescription
                insertNewObjectForEntityForName:@"Book" inManagedObjectContext:managedObjectContext];
        NSString *psTitle      = [pBookInfo objectAtIndex:0];
        NSNumber *pCategory    = [pBookInfo objectAtIndex:1];
        NSNumber *pCountTarget = [pBookInfo objectAtIndex:2];
        [pBook defaultSettingWithCategory:pCategory
                                    title:psTitle
                              countTarget:pCountTarget
                               appSetting:mpAppSetting];
        /* Configure the new setting with information from the location. */
        if(nCount ==0) {
            [mpAppSetting setReadBook:pBook];
        }
        nCount++;
    }
#else
/* if no items is there, insert 3 books */
    /* Create 3 new instances of the Book entity */
    Book *pBookVerse = (Book *)[NSEntityDescription
            insertNewObjectForEntityForName:@"Book" inManagedObjectContext:managedObjectContext];
    [pBookVerse defaultSettingWithCategory:eCategory_Verse appSetting:mpAppSetting];
    Book *pBookBook = (Book *)[NSEntityDescription
            insertNewObjectForEntityForName:@"Book" inManagedObjectContext:managedObjectContext];
    [pBookBook  defaultSettingWithCategory:eCategory_Book  appSetting:mpAppSetting];
    Book *pBookOther = (Book *)[NSEntityDescription
            insertNewObjectForEntityForName:@"Book" inManagedObjectContext:managedObjectContext];
    [pBookOther defaultSettingWithCategory:eCategory_Other appSetting:mpAppSetting];

    /* Configure the new setting with information from the location. */
    //mpAppSetting.selectedBook = pBookVerse;
    [mpAppSetting setReadBook:pBookVerse];
#endif

// Commit the change.
    NSError *error;
    if(![managedObjectContext save:&error]) {
        // Handle the error.
    }
}

//------------------------------------------------------------------------------
/*! update FetchedResultsController to update in-memory database */
- (void)updateFetchedResultsController
{
/* it is a bug fix for SDK 3.2 and 4.0, which are running at Xcode 3.2.3.
 * The bug is that it crashes at the first run.
 */
    self.fetchedResultsController = nil;
    NSError *error;
    if(![[self fetchedResultsController] performFetch:&error]) {
#ifdef DEVELOPMENT_ONLY
        // Update to handle the error appropriately.
        NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        exit(-1);  // Fail
#endif
    }
}

//=============================================================================
// ACTION METHODS
//=============================================================================
//-----------------------------------------------------------------------------
- (IBAction)onCountUp
{
    [self setCountVal:1 isReset:NO];
    [self playSound];
    [self enableCountReset:YES withRestore:NO];    /* enable CountReset */
}

#if 0   /* not used in this project */
//-----------------------------------------------------------------------------
- (IBAction)onCountDown
{
    [self setCountVal:-1 isReset:NO];
    [self playSound];
}
#endif

#if 1
#define kAlertTagCountReset  10

//-----------------------------------------------------------------------------
- (IBAction)onCountReset
{
/* get alert message */
    UIAlertView *pAlertView = [[UIAlertView alloc]
                     initWithTitle:NSLocalizedString(@"Confirm to reset?", @"ask confirmation to reset counter")
                           message:nil
                          delegate:self
                 cancelButtonTitle:NSLocalizedString(@"Yes", @"confirm to reset counter")
                 otherButtonTitles:NSLocalizedString(@"No",  @"do not reset counter"), nil];
    pAlertView.tag = kAlertTagCountReset;
    [pAlertView show];
    [pAlertView release];
}

//---------------------------------------------------------------------------
/*! \fn - (void)alertView:(UIAlertView *)pAlertView clickedButtonAtIndex:(NSInteger)nButtonIndex
 *  \brief UIAlertViewDelegate
 */
- (void)alertView:(UIAlertView *)pAlertView clickedButtonAtIndex:(NSInteger)nButtonIndex
{
    if([pAlertView tag] == kAlertTagCountReset) {
        if(nButtonIndex == 0) {     /* YES */
            [self enableCountReset:NO withRestore:YES];    /* disable CountReset */
            [self setCountVal:0 isReset:YES];
            [self showBtnCountDone:NO];
        }
    }
}

#else
//-----------------------------------------------------------------------------
- (IBAction)onCountReset
{
    [self setCountVal:0 isReset:YES];
    [self showBtnCountDone:NO];
}
#endif

//-----------------------------------------------------------------------------
- (IBAction)onCountRestore
{
    [self enableCountReset:YES withRestore:YES];    /* enable CountReset */
    //[self setCountVal:123 isReset:YES];
}

//-----------------------------------------------------------------------------
- (IBAction)onCountDone
{
    [self onCountUp];
}

//-----------------------------------------------------------------------------
- (IBAction)onBookList
{
#if 0
    BookListViewCtrl *pBookListViewCtrl = [[BookListViewCtrl alloc] init];
#else
    BookListViewCtrl *pBookListViewCtrl = [[BookListViewCtrl alloc]
                initWithNibName:@"BookListView"
                         bundle:nil];
#endif
    pBookListViewCtrl.delegate             = self;
    pBookListViewCtrl.managedObjectContext = managedObjectContext;
    pBookListViewCtrl.mpAppSetting         = mpAppSetting;
    
    pBookListViewCtrl.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;

#if 1
/* bring a NavCtrl to navigate to BookListViewCtrl */
    UINavigationController *navController = [[UINavigationController alloc]
                initWithRootViewController:pBookListViewCtrl];
    [pBookListViewCtrl release];
    [self presentModalViewController:navController animated:YES];
    [navController release];
#else
/* simply present BookListViewCtrl in ModalViewController */
    [self presentModalViewController:pBookListViewCtrl animated:YES];
    [pBookListViewCtrl release];
#endif
}


//-----------------------------------------------------------------------------
- (IBAction)onShowInfo
{
#if 0
    InfoViewCtrl *pInfoViewCtrl = [[InfoViewCtrl alloc]
                            initWithNibName:@"InfoView" bundle:nil];
    pInfoViewCtrl.delegate = self;

/* simply present InfoViewCtrl in ModalViewController */
    pInfoViewCtrl.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
    [self presentModalViewController:pInfoViewCtrl animated:YES];
    [pInfoViewCtrl release];
#endif
}


//-----------------------------------------------------------------------------
- (IBAction)onShowOption
{
#define USE_INTERFACE_BUILDER
#ifdef USE_INTERFACE_BUILDER
    OptionViewCtrl *pOptionViewCtrl = [[OptionViewCtrl alloc]
                initWithNibName:@"OptionView"
                         bundle:nil];
#else
    OptionViewCtrl *pOptionViewCtrl = [[OptionViewCtrl alloc] init];
#endif
    pOptionViewCtrl.optionViewCtrlDelegate = self;
    pOptionViewCtrl.managedObjectContext   = managedObjectContext;    /* TODO: remove */
    pOptionViewCtrl.mpAppSetting           = mpAppSetting;
    //NSLog(@"title=%@", mpAppSetting.selectedBook.title);
    pOptionViewCtrl.modalTransitionStyle   = UIModalTransitionStyleCrossDissolve;

#if 1
/* bring a NavCtrl to navigate to OptionViewCtrl */
    UINavigationController *navController = [[UINavigationController alloc]
                initWithRootViewController:pOptionViewCtrl];
    [pOptionViewCtrl release];
    [self presentModalViewController:navController animated:YES];
    [navController release];
#else
/* simply present OptionViewCtrl in ModalViewController */
    [self presentModalViewController:pOptionViewCtrl animated:YES];
    [pOptionViewCtrl release];
#endif
}

//=============================================================================
// PROTOCOL METHODS
//=============================================================================
#pragma mark

#if 0
#pragma mark InfoViewCtrlDelegate
//-----------------------------------------------------------------------------
- (void)infoViewCtrlDidFinish:(InfoViewCtrl *)controller
{
    [self dismissModalViewControllerAnimated:YES];
}
#endif

#pragma mark OptionViewCtrlDelegate
//-----------------------------------------------------------------------------
- (void)optionViewCtrl:(OptionViewCtrl *)pViewCtrl didFinishWithChangeFlag:(int)nChangeFlag
{
#if 0   /* no dynamic sorting */
/* take care of SortingOrder only */
    if(nChangeFlag & eOptionFlag_SortingKey) {
        /* reset fetchedResultsController */
        self.fetchedResultsController = nil;

        /* Executes the fetch request on the store to get objects.
         * Returns YES if successful or NO (and an error) if a problem occurred. 
         * An error is returned if the fetch request specified doesn't include
         * a sort descriptor that uses sectionNameKeyPath.
         * After executing this method, the fetched objects can be accessed
         * with the property 'fetchedObjects'
         */
        NSError *error;
        if(![[self fetchedResultsController] performFetch:&error]) {
            // Update to handle the error appropriately.
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            exit(-1);  // Fail
        }
    }
#endif

/* dismiss Modal View controller */
    [self dismissModalViewControllerAnimated:YES];
}

#pragma mark BookListViewCtrlDelegate
//-----------------------------------------------------------------------------
- (void)bookListViewCtrlDidFinish:(BookListViewCtrl *)controller isUpdated:(bool)bIsUpdated
{
/*  update fetchedResultsController, if BookListViewCtrl adds or deletes books. */
    if(bIsUpdated) {
        [self updateFetchedResultsController]; 
    }
    
    [self dismissModalViewControllerAnimated:YES];
}

#pragma mark UIPickerViewDataSource
//-----------------------------------------------------------------------------
/*! Required method: returns the number of 'columns' to display. */
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    int nNumComp = 1;

/* case by case */
    if(pickerView == xpPickerBook) {
        nNumComp = 1;
    } else if(pickerView == xpPickerCounter) {
        nNumComp = kCounterNumComp;
    }

/* return */
    return nNumComp;
}

//-----------------------------------------------------------------------------
/*! Required method: returns the # of rows in each component. */
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    int nNumRow = 1;

/* case by case */
    if(pickerView == xpPickerBook) {
#if 1
        nNumRow = [[fetchedResultsController fetchedObjects] count];
#else
        id <NSFetchedResultsSectionInfo> sectionInfo = [[fetchedResultsController sections]
                                                         objectAtIndex:0];
        nNumRow = [sectionInfo numberOfObjects];
#endif
    } else if(pickerView == xpPickerCounter) {
        nNumRow = kCounterNumRowInView;
    }

/* return */
    return nNumRow;
}

#pragma mark UIPickerViewDelegate
#if 0
//-----------------------------------------------------------------------------
/*! Optional method: returns width of column and height of row for each component. */
- (CGFloat)pickerView:(UIPickerView *)pickerView widthForComponent:(NSInteger)component
{
    float fWidth = 10.0;

/* case by case */
    if(pickerView == xpPickerBook) {
    } else if(pickerView == xpPickerCounter) {
    }

/* return */
    return fWidth;
}
#endif

//-----------------------------------------------------------------------------
/*! Optional method: returns width of column and height of row for each component. */
- (CGFloat)pickerView:(UIPickerView *)pickerView rowHeightForComponent:(NSInteger)component
{
    float fHeight = 10.0;

/* case by case */
    if(pickerView == xpPickerBook) {
        fHeight = 36;
    } else if(pickerView == xpPickerCounter) {
        fHeight = 68;
    }

/* return */
    return fHeight;
}

//-----------------------------------------------------------------------------
/*! These methods return either a plain UIString, or a view (e.g UILabel)
 * to display the row for the component.
 * For the view versions, we cache any hidden and thus unused views and pass them back for reuse.
 * If you return back a different object, the old one will be released.
 * the view will be centered in the row rect
 */
#if 0
/* optional method */
- (NSString *)pickerView:(UIPickerView *)pickerView
   titleForRow:(NSInteger)row
   forComponent:(NSInteger)component
{
    NSString *psTitle = nil;

/* case by case */
    if(pickerView == xpPickerBook) {
    } else if(pickerView == xpPickerCounter) {
        psTitle = [NSString stringWithFormat:@"%d", row];
    }

/* return */
    return psTitle;
}
#endif

//-----------------------------------------------------------------------------
/* optional method */
- (UIView *)pickerView:(UIPickerView *)pickerView
            viewForRow:(NSInteger)row
          forComponent:(NSInteger)component
           reusingView:(UIView *)view
{
    UIView *pView = nil;

/* case by case */
    if(pickerView == xpPickerBook) {
        UILabel *pLabel;
        if((view == nil) || ![view isKindOfClass:[UILabel class]]) {   /* create a new one */
            pLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, 280, 36)];
            pLabel.font = [UIFont fontWithName:@"Verdana" size:24];
            pLabel.textColor = [UIColor blackColor];    //[UIColor yellowColor];
            pLabel.backgroundColor = [UIColor clearColor];
            [pLabel autorelease];
        } else {
            pLabel = (UILabel *)view;
        }

        /* Get the book corresponding to the current index path and configure the table view cell. */
        Book *pBook = (Book*)[[fetchedResultsController fetchedObjects] objectAtIndex:row];
        /* Configure the cell to show the book's title */
        pLabel.text = pBook.title;
        pView = (UIView *)pLabel;
        //NSLog(@"row=%d, title=%@", row, pBook.title);

    } else if(pickerView == xpPickerCounter) {
        const int nWhichImage = row % kCounterNumRow;
        UIImage *pImage = [mpImageArray objectAtIndex:nWhichImage];
        UIImageView *pImageView;
        if((view == nil) || ![view isKindOfClass:[UIImageView class]]) {   /* create a new one */
            //NSLog(@">> comp=%d, row=%d", component, row);
            pImageView = [[UIImageView alloc] initWithImage:pImage];
            pImageView.backgroundColor = [UIColor blackColor];
            [pImageView autorelease];
        } else {
            //NSLog(@"## comp=%d, row=%d, %d", component, row, [view isKindOfClass:[UIImageView class]]);
            pImageView = (UIImageView*)view;
            pImageView.image = pImage;
        }
        pView = (UIView *)pImageView;
    }

/* return */
    return pView;
}

//-----------------------------------------------------------------------------
/* optional method */
- (void)pickerView:(UIPickerView *)pickerView
      didSelectRow:(NSInteger)row
       inComponent:(NSInteger)component
{
/* case by case */
    if(pickerView == xpPickerBook) {
        [self enableCountReset:YES withRestore:NO];    /* enable CountReset */
        Book *pSelectedBook = (Book*)[[fetchedResultsController fetchedObjects] objectAtIndex:row];
        if(mpReadBook != pSelectedBook) {
            /* update the count of the old selected Book */
            [self updateReadBook:pSelectedBook];
            [self displayTargetCountOfBook];
        }
    } else if(pickerView == xpPickerCounter) {
    }
}

//==============================================================================
// CoreData Fetch
//==============================================================================
#pragma mark -
#pragma mark Fetched results controller (NSFetchedResultsControllerDelegate)

/* In general, NSFetchedResultsController is designed to respond to changes
 * at the model layer. If you allow a user to reorder table rows,
 * then your implementation of the delegate methods must take this into account.
 *
 * In the MainViewCtrl case, it does not add, remove, or update the fetched results,
 * unless Book is updated by the BookListViewCtrl.
 * controllerDidChangeContent handles the changes by the BookListViewCtrl.
 */
//------------------------------------------------------------------------------
/*! Returns the fetched results controller.
 * Creates and configures the controller if necessary.
 */
- (NSFetchedResultsController *)fetchedResultsController
{
/* Creates and configures the controller if necessary. */
    if(fetchedResultsController != nil) {
        return fetchedResultsController;
    }

/* sortingKey */
#if 1   /* no dynamic sorting */
    NSString *pSortingKeyInString = @"title";
    BOOL bSortingKeyIsAscending   = YES;
#else
    //NSLog(@"%d", [mpAppSetting.sortingKey intValue]);
    NSString *pSortingKeyInString = [mpAppSetting sortingKeyInString];
    BOOL bSortingKeyIsAscending   = [mpAppSetting sortingKeyIsAscending];
#endif

/* Create and configure a fetch request with the Book entity. */
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Book"
                                              inManagedObjectContext:managedObjectContext];
    [fetchRequest setEntity:entity];

/* Create the sort descriptors array to fetchRequest. */
    NSSortDescriptor *categoryDescriptor = [[NSSortDescriptor alloc] initWithKey:@"category" ascending:YES];
    NSSortDescriptor *titleDescriptor = [[NSSortDescriptor alloc]
                                            initWithKey:pSortingKeyInString
                                              ascending:bSortingKeyIsAscending];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:categoryDescriptor, titleDescriptor, nil];
    [categoryDescriptor release];
    [titleDescriptor release];

    [fetchRequest setSortDescriptors:sortDescriptors];
    [sortDescriptors release];

/* Create and initialize the fetch results controller (an instance of NSFetchedResultsController)
 * fetchRequest       - the fetch request used to get the objects.
 *                      It's expected that the sort descriptor used in
 *                      the request groups the objects into sections.
 * context            - the context that will hold the fetched objects
 * sectionNameKeyPath - keypath on resulting objects that returns the section name.
 *                      This will be used to pre-compute the section information.
 * cacheName          - Pre computed section info is cached to a private directory under this name.
 *                      If there is pre computed section info stored under this name,
 *                      it is checked to see if it matches the fetchRequest.
 *                      If the cached info doesn't match the request,
 *                      it's deleted and recomputed when the fetch happens.
 */
    NSFetchedResultsController *aFetchedResultsController = [[NSFetchedResultsController alloc]
                        initWithFetchRequest:fetchRequest
                        managedObjectContext:managedObjectContext
                          sectionNameKeyPath:nil            /* single section */
                                   cacheName:@"MainViewCtrl"];
    self.fetchedResultsController = aFetchedResultsController;
    /* set delegate */
    fetchedResultsController.delegate = self;
    [aFetchedResultsController release];
    [fetchRequest release];

/* return */
    return fetchedResultsController;
}

//=============================================================================
@end    /* MainViewCtrl */

