/*!
 * \file  OptionViewCtrl.m
 *       Implement Option View Controller for BuddhaCounter project
 *
 * The table view controller responsible for displaying the
 *           list of books, supporting additional functionality:
 * - Drill-down to display more information about a selected book using
 *   an instance of DetailViewCtrl;
 * - Addition of new new books;
 * - Deletion of existing books using UITableView's
 *   tableView:commitEditingStyle:forRowAtIndexPath: method.
 */
/***************************************************************************/
/* Copyright (c) 2010 AuroraWand LLC                                       */
/* All rights reserved.                                                    */
/*                                                                         */
/* No part of this document may be copied, reproduced,                     */
/* or transmitted in any form without the prior written                    */
/* consent of AuroraWand LLC, San Jose, CA                                 */
/*                                                                         */
/* Created by LinCheng Wang on 05/25/10                                    */
/***************************************************************************/

//=============================================================================
// COMPILATION OPTIONS
//=============================================================================
//#define DEBUG_LEVEL     /* to turn on message utility by UtilMsg.h */

#define TABLE_VIEW_CELL_DESIGN
#define USE_INTERFACE_BUILDER

//=============================================================================
// HEADER FILES
//=============================================================================
#import "OptionViewCtrl.h"
#import "BookDetailViewCtrl.h"

#import "InfoViewCtrl.h"
#import "FeatureViewCtrl.h"
#import "HelpViewCtrl.h"

#import "AppModel.h"
#import "AppSetting.h"

//=============================================================================
// MACROS & INLINE FUNCTIONS
//=============================================================================

//=============================================================================
// TYPEDEFS and DATA STRUCTURES
//=============================================================================

//=============================================================================
// PRIVATE METHODS
//=============================================================================
@interface OptionViewCtrl (Private)
- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath;
@end

//=============================================================================
// CLASS IMPLEMENTATION
//=============================================================================
@implementation OptionViewCtrl

@synthesize optionViewCtrlDelegate;
@synthesize fetchedResultsController, managedObjectContext;
@synthesize mpAppSetting;

@synthesize xpBgView;
@synthesize xpCountingOrder, xpSortingKey, xpPlaySound;
@synthesize xpBtnShowInfo, xpBtnShowFeature, xpBtnShowHelp;

//=============================================================================
// METHODS
//=============================================================================
#pragma mark -
#pragma mark View lifecycle

//-----------------------------------------------------------------------------
- (void)viewDidLoad
{
    [super viewDidLoad];

/* make up View */
    /* Set the title. */
    self.title = NSLocalizedString(@"Options", @"the view title of setting options view");
    //self.view.backgroundColor = [UIColor viewFlipsideBackgroundColor];  /* TODO */

    /* Set up the edit and add buttons.
     * The default button action invokes the setEditing:animated: method.
     */
    //self.navigationItem.leftBarButtonItem = self.editButtonItem;

    /* Configure the Done button. It is an action to back to previous view controller. */
    UIBarButtonItem *pBtnDone = [[UIBarButtonItem alloc]
        initWithBarButtonSystemItem:UIBarButtonSystemItemDone
                             target:self
                             action:@selector(onDone)];
    self.navigationItem.leftBarButtonItem = pBtnDone;
    [pBtnDone release];
    
/* current settings */
    xpCountingOrder.selectedSegmentIndex = [mpAppSetting.bCountingOrder intValue];
    xpSortingKey.selectedSegmentIndex    = [mpAppSetting.sortingKey     intValue];
    xpPlaySound.selectedSegmentIndex     = ([mpAppSetting.bPlaySound    intValue] == 0);
}

#if 0
//-----------------------------------------------------------------------------
- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}
#endif

//-----------------------------------------------------------------------------
- (void)viewDidUnload
{
/* Release any properties that are loaded in viewDidLoad or can be recreated lazily. */
    self.fetchedResultsController = nil;

    self.xpBgView          = nil;
    self.xpCountingOrder   = nil;
    self.xpSortingKey      = nil;
    self.xpPlaySound       = nil;
    self.xpBtnShowInfo     = nil;
    self.xpBtnShowFeature  = nil;
    self.xpBtnShowHelp     = nil;
}

#pragma mark -
#pragma mark Memory management

//-----------------------------------------------------------------------------
- (void)dealloc
{
    [fetchedResultsController release];
    [managedObjectContext release];

    [xpBgView          release];
    [xpCountingOrder   release];
    [xpSortingKey      release];
    [xpPlaySound       release];
    [xpBtnShowInfo     release];
    [xpBtnShowFeature  release];
    [xpBtnShowHelp     release];

    [super dealloc];
}

//=============================================================================
// ACTION METHODS
//=============================================================================

//-----------------------------------------------------------------------------
- (IBAction)onDone
{
/* ChangeFlag */
    int nChangeFlag = 0;

    if([mpAppSetting.bCountingOrder intValue] != xpCountingOrder.selectedSegmentIndex) {
        mpAppSetting.bCountingOrder = [NSNumber numberWithInt:xpCountingOrder.selectedSegmentIndex];
        nChangeFlag |= eOptionFlag_CountingOrder;
    }
    if([mpAppSetting.sortingKey     intValue] != xpSortingKey.selectedSegmentIndex) {
        mpAppSetting.sortingKey = [NSNumber numberWithInt:xpSortingKey.selectedSegmentIndex];
        nChangeFlag |= eOptionFlag_SortingKey;
    }
    if([mpAppSetting.bPlaySound intValue] != (xpPlaySound.selectedSegmentIndex == 0)) {
        mpAppSetting.bPlaySound = [NSNumber numberWithInt:(xpPlaySound.selectedSegmentIndex == 0)];
        nChangeFlag |= eOptionFlag_CountingOrder;
    }

/* report to delegate */
    [self.optionViewCtrlDelegate optionViewCtrl:self didFinishWithChangeFlag:nChangeFlag];	
}

//-----------------------------------------------------------------------------
- (IBAction)onCountingOrder
{
}

#if 0
//-----------------------------------------------------------------------------
- (IBAction)onSortingKey
{
}
#endif

//-----------------------------------------------------------------------------
- (IBAction)onPlaySound
{
}

//-----------------------------------------------------------------------------
- (IBAction)onShowInfo:(id)pSender;
{
    UIButton *pBtn = (UIButton*)pSender;

    UIViewController    *pViewCtrl = nil;
    if(pBtn == xpBtnShowInfo) {
        pViewCtrl = [[InfoViewCtrl alloc] initWithNibName:@"InfoView" bundle:nil];
    } else if(pBtn == xpBtnShowFeature) {
        pViewCtrl = [[FeatureViewCtrl alloc] initWithNibName:@"FeatureView" bundle:nil];
    } else if(pBtn == xpBtnShowHelp) {
        pViewCtrl = [[HelpViewCtrl alloc] initWithNibName:@"HelpView" bundle:nil];
    }
    
    /* simply present InfoViewCtrl in ModalViewController */
    pViewCtrl.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
    //[self presentModalViewController:pViewCtrl animated:YES];
    [self.navigationController pushViewController:pViewCtrl animated:YES];
    [pViewCtrl release];
}

//=============================================================================
@end

